#!/usr/bin/env python
# coding=utf-8

"""
Synthesize several text fragments,
producing a WAV audio file.
"""

from __future__ import absolute_import
from __future__ import print_function
import sys

from aeneas.language import Language
from aeneas.synthesizer import Synthesizer
from aeneas.tools.abstract_cli_program import AbstractCLIProgram
import aeneas.globalconstants as gc
import aeneas.globalfunctions as gf

__author__ = "Alberto Pettarin"
__copyright__ = """
    Copyright 2012-2013, Alberto Pettarin (www.albertopettarin.it)
    Copyright 2013-2015, ReadBeyond Srl   (www.readbeyond.it)
    Copyright 2015-2016, Alberto Pettarin (www.albertopettarin.it)
    """
__license__ = "GNU AGPL 3"
__version__ = "1.4.1"
__email__ = "aeneas@readbeyond.it"
__status__ = "Production"

class SynthesizeTextCLI(AbstractCLIProgram):
    """
    Synthesize several text fragments,
    producing a WAV audio file.
    """
    OUTPUT_FILE = "output/synthesized.wav"
    TEXT_FILE_PARSED = gf.relative_path("res/parsed.txt", __file__)
    TEXT_FILE_PLAIN = gf.relative_path("res/plain.txt", __file__)
    TEXT_FILE_SUBTITLES = gf.relative_path("res/subtitles.txt", __file__)
    TEXT_FILE_UNPARSED = gf.relative_path("res/unparsed.xhtml", __file__)

    NAME = gf.file_name_without_extension(__file__)

    HELP = {
        "description": u"Synthesize several text fragments.",
        "synopsis": [
            u"list 'fragment 1|fragment 2|...|fragment N' LANGUAGE OUTPUT_FILE",
            u"[parsed|plain|subtitles|unparsed] TEXT_FILE LANGUAGE OUTPUT_FILE"
        ],
        "examples": [
            u"list 'From|fairest|creatures|we|desire|increase' en %s" % (OUTPUT_FILE),
            u"parsed %s en %s" % (TEXT_FILE_PARSED, OUTPUT_FILE),
            u"plain %s en %s" % (TEXT_FILE_PLAIN, OUTPUT_FILE),
            u"subtitles %s en %s" % (TEXT_FILE_SUBTITLES, OUTPUT_FILE),
            u"unparsed %s en %s --id-regex=f[0-9]*" % (TEXT_FILE_UNPARSED, OUTPUT_FILE),
            u"unparsed %s en %s --class-regex=ra" % (TEXT_FILE_UNPARSED, OUTPUT_FILE),
            u"unparsed %s en %s --id-regex=f[0-9]* --sort=numeric" % (TEXT_FILE_UNPARSED, OUTPUT_FILE),
            u"plain %s en %s --start=5" % (TEXT_FILE_PLAIN, OUTPUT_FILE),
            u"plain %s en %s --end=10" % (TEXT_FILE_PLAIN, OUTPUT_FILE),
            u"plain %s en %s --start=5 --end=10" % (TEXT_FILE_PLAIN, OUTPUT_FILE),
            u"plain %s en %s --backwards" % (TEXT_FILE_PLAIN, OUTPUT_FILE),
            u"plain %s en %s --quit-after=10.0" % (TEXT_FILE_PLAIN, OUTPUT_FILE),
        ],
        "options": [
            u"--class-regex=REGEX : extract text from elements with class attribute matching REGEX (unparsed)",
            u"--end=INDEX : slice the text file until fragment INDEX",
            u"--id-format=FMT : use FMT for generating text id attributes (subtitles, plain)",
            u"--id-regex=REGEX : extract text from elements with id attribute matching REGEX (unparsed)",
            u"--quit-after=DUR : synthesize fragments until DUR seconds or the end of text is reached",
            u"--sort=ALGORITHM : sort the matched element id attributes using ALGORITHM (lexicographic, numeric, unsorted)",
            u"--start=INDEX : slice the text file from fragment INDEX",
            u"-b, --backwards : synthesize from the last fragment to the first one",
        ]
    }

    def perform_command(self):
        """
        Perform command and return the appropriate exit code.

        :rtype: int
        """
        if len(self.actual_arguments) < 4:
            return self.print_help()
        text_format = gf.safe_unicode(self.actual_arguments[0])
        if text_format == u"list":
            text = gf.safe_unicode(self.actual_arguments[1])
        elif text_format in [u"parsed", u"plain", u"subtitles", u"unparsed"]:
            text = self.actual_arguments[1]
            if not self.check_input_file(text):
                return self.ERROR_EXIT_CODE
        else:
            return self.print_help()

        id_regex = self.has_option_with_value(u"--id-regex")
        class_regex = self.has_option_with_value(u"--class-regex")
        sort = self.has_option_with_value(u"--sort")
        backwards = self.has_option([u"-b", u"--backwards"])
        quit_after = gf.safe_float(self.has_option_with_value(u"--quit-after"), None)
        start_fragment = gf.safe_int(self.has_option_with_value(u"--start"), None)
        end_fragment = gf.safe_int(self.has_option_with_value(u"--end"), None)
        parameters = {
            gc.PPN_JOB_IS_TEXT_UNPARSED_ID_REGEX : id_regex,
            gc.PPN_JOB_IS_TEXT_UNPARSED_CLASS_REGEX : class_regex,
            gc.PPN_JOB_IS_TEXT_UNPARSED_ID_SORT : sort
        }
        if (text_format == u"unparsed") and (id_regex is None) and (class_regex is None):
            self.print_error(u"You must specify --id-regex and/or --class-regex for unparsed format")
            return self.ERROR_EXIT_CODE

        language = gf.safe_unicode(self.actual_arguments[2])
        if (not language in Language.ALLOWED_VALUES) and (not self.rconf["allow_unlisted_languages"]):
            self.print_error(u"Language '%s' is not supported" % (language))
            return self.ERROR_EXIT_CODE

        output_file_path = self.actual_arguments[3]
        if not self.check_output_file(output_file_path):
            return self.ERROR_EXIT_CODE

        text_file = self.get_text_file(text_format, text, parameters)
        if text_file is None:
            self.print_error(u"Unable to build a TextFile from the given parameters")
            return self.ERROR_EXIT_CODE
        text_file.set_language(language)
        self.print_info(u"Read input text with %d fragments" % (len(text_file)))
        if start_fragment is not None:
            self.print_info(u"Slicing from index %d" % (start_fragment))
        if end_fragment is not None:
            self.print_info(u"Slicing to index %d" % (end_fragment))
        text_slice = text_file.get_slice(start_fragment, end_fragment)
        self.print_info(u"Synthesizing %d fragments" % (len(text_slice)))

        if quit_after is not None:
            self.print_info(u"Stop synthesizing upon reaching %.3f seconds" % (quit_after))

        try:
            synt = Synthesizer(rconf=self.rconf, logger=self.logger)
            synt.synthesize(
                text_slice,
                output_file_path,
                quit_after=quit_after,
                backwards=backwards
            )
            self.print_info(u"Created file '%s'" % output_file_path)
            return self.NO_ERROR_EXIT_CODE
        except Exception as exc:
            self.print_error(u"An unexpected Exception occurred while synthesizing text:")
            self.print_error(u"%s" % exc)

        return self.ERROR_EXIT_CODE


def main():
    """
    Execute program.
    """
    SynthesizeTextCLI().run(arguments=sys.argv)

if __name__ == '__main__':
    main()






#!/usr/bin/env python
# coding=utf-8

"""
Read audio file properties using the ``ffprobe`` wrapper.
"""

from __future__ import absolute_import
from __future__ import print_function
import sys

from aeneas.ffprobewrapper import FFPROBEParsingError
from aeneas.ffprobewrapper import FFPROBEPathError
from aeneas.ffprobewrapper import FFPROBEUnsupportedFormatError
from aeneas.ffprobewrapper import FFPROBEWrapper
from aeneas.tools.abstract_cli_program import AbstractCLIProgram
import aeneas.globalfunctions as gf

__author__ = "Alberto Pettarin"
__copyright__ = """
    Copyright 2012-2013, Alberto Pettarin (www.albertopettarin.it)
    Copyright 2013-2015, ReadBeyond Srl   (www.readbeyond.it)
    Copyright 2015-2016, Alberto Pettarin (www.albertopettarin.it)
    """
__license__ = "GNU AGPL 3"
__version__ = "1.4.1"
__email__ = "aeneas@readbeyond.it"
__status__ = "Production"

class FFPROBEWrapperCLI(AbstractCLIProgram):
    """
    Read audio file properties using the ``ffprobe`` wrapper.
    """
    AUDIO_FILE = gf.relative_path("res/audio.mp3", __file__)

    NAME = gf.file_name_without_extension(__file__)

    HELP = {
        "description": u"Read audio file properties using the ffprobe wrapper.",
        "synopsis": [
            u"AUDIO_FILE"
        ],
        "examples": [
            u"%s" % (AUDIO_FILE)
        ]
    }

    def perform_command(self):
        """
        Perform command and return the appropriate exit code.

        :rtype: int
        """
        if len(self.actual_arguments) < 1:
            return self.print_help()
        audio_file_path = self.actual_arguments[0]

        if not self.check_input_file(audio_file_path):
            return self.ERROR_EXIT_CODE

        try:
            prober = FFPROBEWrapper(rconf=self.rconf, logger=self.logger)
            dictionary = prober.read_properties(audio_file_path)
            for key in sorted(dictionary.keys()):
                self.print_generic(u"%s %s" % (key, dictionary[key]))
            return self.NO_ERROR_EXIT_CODE
        except FFPROBEPathError:
            self.print_error(u"Unable to call the ffprobe executable '%s'" % (self.rconf["ffprobe_path"]))
            self.print_error(u"Make sure the path to ffprobe is correct")
        except (FFPROBEUnsupportedFormatError, FFPROBEParsingError):
            self.print_error(u"Cannot read properties of file '%s'" % (audio_file_path))
            self.print_error(u"Make sure the input file has a format supported by ffprobe")

        return self.ERROR_EXIT_CODE



def main():
    """
    Execute program.
    """
    FFPROBEWrapperCLI().run(arguments=sys.argv)

if __name__ == '__main__':
    main()






#!/usr/bin/env python
# coding=utf-8

"""
Download files from various Web sources.
"""

from __future__ import absolute_import
from __future__ import print_function

from aeneas.logger import Logger
from aeneas.runtimeconfiguration import RuntimeConfiguration
import aeneas.globalfunctions as gf

__author__ = "Alberto Pettarin"
__copyright__ = """
    Copyright 2012-2013, Alberto Pettarin (www.albertopettarin.it)
    Copyright 2013-2015, ReadBeyond Srl   (www.readbeyond.it)
    Copyright 2015-2016, Alberto Pettarin (www.albertopettarin.it)
    """
__license__ = "GNU AGPL v3"
__version__ = "1.4.1"
__email__ = "aeneas@readbeyond.it"
__status__ = "Production"

class Downloader(object):
    """
    Download files from various Web sources.

    :param rconf: a runtime configuration. Default: ``None``, meaning that
                  default settings will be used.
    :type  rconf: :class:`aeneas.runtimeconfiguration.RuntimeConfiguration`
    :param logger: the logger object
    :type  logger: :class:`aeneas.logger.Logger`
    """

    TAG = u"Downloader"

    def __init__(self, rconf=None, logger=None):
        self.logger = logger or Logger()
        self.rconf = rconf or RuntimeConfiguration()

    def _log(self, message, severity=Logger.DEBUG):
        """ Log """
        self.logger.log(message, severity, self.TAG)

    def audio_from_youtube(
            self,
            source_url,
            download=True,
            output_file_path=None,
            preferred_index=None,
            largest_audio=True,
            preferred_format=None
    ):
        """
        Download an audio stream from a YouTube video,
        and save it to file.

        If ``download`` is ``False``, return the list
        of available audiostreams but do not download.

        Otherwise, download the audio stream best matching
        the provided parameters, as follows.
        If ``preferred_index`` is not ``None``,
        download the audio stream at that index.
        If ``largest_audio`` is ``True``,
        download the largest audiostream;
        otherwise, download the smallest audiostream.
        If ``preferred_format`` is not ``None``,
        download the audiostream having that format.
        The latter option works in combination with ``largest_audio``.

        Return the path of the downloaded file.

        :param source_url: the URL of the YouTube video
        :type  source_url: string (url)
        :param download: if ``True``, download the audio stream
                         best matching ``preferred_index`` or ``preferred_format``
                         and ``largest_audio``;
                         if ``False``, return the list of available audio streams
        :type  download: bool
        :param output_file_path: the path where the downloaded audio should be saved;
                                 if ``None``, create a temporary file
        :type  output_file_path: string (path)
        :param preferred_index: preferably download this audio stream
        :type  preferred_index: int
        :param largest_audio: if ``True``, download the largest audio stream available;
                              if ``False``, download the smallest one.
        :type  largest_audio: bool
        :param preferred_format: preferably download this audio format
        :type  preferred_format: string

        :rtype: string (path) or list of pafy audio streams

        :raise ImportError: if ``pafy`` is not installed
        :raise OSError: if ``output_file_path`` cannot be written
        :raise ValueError: if ``source_url`` is not a valid YouTube URL
        """
        def select_audiostream(audiostreams):
            """ Select the audiostream best matching the given parameters. """
            if preferred_index is not None:
                if preferred_index in range(len(audiostreams)):
                    self._log([u"Selecting audiostream with index %d", preferred_index])
                    return audiostreams[preferred_index]
                else:
                    self._log([u"Audio stream index %d not allowed", preferred_index], Logger.WARNING)
                    self._log(u"Ignoring the requested audio stream index", Logger.WARNING)
            # filter by preferred format
            streams = audiostreams
            if preferred_format is not None:
                self._log([u"Filtering audiostreams by preferred format %s", preferred_format])
                streams = [audiostream for audiostream in streams if audiostream.extension == preferred_format]
                if len(streams) < 1:
                    self._log([u"No audiostream with preferred format %s", preferred_format])
                    streams = audiostreams
            # sort by size
            streams = sorted([(audio.get_filesize(), audio) for audio in streams])
            if largest_audio:
                self._log(u"Selecting largest audiostream")
                selected = streams[-1][1]
            else:
                self._log(u"Selecting smallest audiostream")
                selected = streams[0][1]
            return selected

        try:
            import pafy
        except ImportError as exc:
            self._log(u"pafy is not installed", Logger.CRITICAL)
            raise exc

        try:
            video = pafy.new(source_url)
        except (IOError, OSError, ValueError) as exc:
            self._log([u"The specified source URL '%s' is not a valid YouTube URL", source_url], Logger.CRITICAL)
            raise ValueError("The specified source URL is not a valid YouTube URL")

        if not download:
            self._log(u"Returning the list of audio streams")
            return video.audiostreams

        output_path = output_file_path
        if output_file_path is None:
            self._log(u"output_path is None: creating temp file")
            handler, output_path = gf.tmp_file(root=self.rconf["tmp_path"])
        else:
            if not gf.file_can_be_written(output_path):
                self._log([u"Path '%s' cannot be written (wrong permissions?)", output_path], Logger.CRITICAL)
                raise OSError("Path '%s' cannot be written (wrong permissions?)" % output_path)

        audiostream = select_audiostream(video.audiostreams)
        if output_file_path is None:
            gf.delete_file(handler, output_path)
            output_path += "." + audiostream.extension

        self._log([u"output_path is '%s'", output_path])
        self._log(u"Downloading...")
        audiostream.download(filepath=output_path, quiet=True)
        self._log(u"Downloading... done")
        return output_path








#!/usr/bin/env python
# coding=utf-8

"""
Extract a list of speech intervals from the given audio file,
using the MFCC energy-based VAD algorithm.
"""

from __future__ import absolute_import
from __future__ import print_function
import io
import sys

from aeneas.audiofile import AudioFileMonoWAVE
from aeneas.audiofile import AudioFileUnsupportedFormatError
from aeneas.ffmpegwrapper import FFMPEGWrapper
from aeneas.tools.abstract_cli_program import AbstractCLIProgram
from aeneas.vad import VAD
import aeneas.globalfunctions as gf

__author__ = "Alberto Pettarin"
__copyright__ = """
    Copyright 2012-2013, Alberto Pettarin (www.albertopettarin.it)
    Copyright 2013-2015, ReadBeyond Srl   (www.readbeyond.it)
    Copyright 2015-2016, Alberto Pettarin (www.albertopettarin.it)
    """
__license__ = "GNU AGPL 3"
__version__ = "1.4.1"
__email__ = "aeneas@readbeyond.it"
__status__ = "Production"

class RunVADCLI(AbstractCLIProgram):
    """
    Extract a list of speech intervals from the given audio file,
    using the MFCC energy-based VAD algorithm.
    """
    INPUT_FILE = gf.relative_path("res/audio.mp3", __file__)
    OUTPUT_BOTH = "output/both.txt"
    OUTPUT_NONSPEECH = "output/nonspeech.txt"
    OUTPUT_SPEECH = "output/speech.txt"
    MODES = [u"both", u"nonspeech", u"speech"]

    NAME = gf.file_name_without_extension(__file__)

    HELP = {
        "description": u"Extract a list of speech intervals using the MFCC energy-based VAD.",
        "synopsis": [
            u"AUDIO_FILE [%s] [OUTPUT_FILE]" % (u"|".join(MODES))
        ],
        "examples": [
            u"%s both %s" % (INPUT_FILE, OUTPUT_BOTH),
            u"%s nonspeech %s" % (INPUT_FILE, OUTPUT_NONSPEECH),
            u"%s speech %s" % (INPUT_FILE, OUTPUT_SPEECH)
        ]
    }

    def perform_command(self):
        """
        Perform command and return the appropriate exit code.

        :rtype: int
        """
        if len(self.actual_arguments) < 2:
            return self.print_help()
        audio_file_path = self.actual_arguments[0]
        mode = self.actual_arguments[1]
        if mode not in [u"speech", u"nonspeech", u"both"]:
            return self.print_help()
        output_file_path = None
        if len(self.actual_arguments) >= 3:
            output_file_path = self.actual_arguments[2]

        self.check_c_extensions("cmfcc")
        if not self.check_input_file(audio_file_path):
            return self.ERROR_EXIT_CODE
        if (output_file_path is not None) and (not self.check_output_file(output_file_path)):
            return self.ERROR_EXIT_CODE

        tmp_handler, tmp_file_path = gf.tmp_file(suffix=u".wav", root=self.rconf["tmp_path"])
        try:
            self.print_info(u"Converting audio file to mono...")
            converter = FFMPEGWrapper(rconf=self.rconf, logger=self.logger)
            converter.convert(audio_file_path, tmp_file_path)
            self.print_info(u"Converting audio file to mono... done")
        except OSError:
            self.print_error(u"Cannot convert audio file '%s'" % audio_file_path)
            self.print_error(u"Check that its format is supported by ffmpeg")
            return self.ERROR_EXIT_CODE

        try:
            self.print_info(u"Extracting MFCCs...")
            audiofile = AudioFileMonoWAVE(tmp_file_path, rconf=self.rconf, logger=self.logger)
            audiofile.extract_mfcc()
            self.print_info(u"Extracting MFCCs... done")
        except (AudioFileUnsupportedFormatError, OSError):
            self.print_error(u"Cannot read the converted WAV file '%s'" % tmp_file_path)
            return self.ERROR_EXIT_CODE

        self.print_info(u"Executing VAD...")
        vad = VAD(audiofile.audio_mfcc, audiofile.audio_length, rconf=self.rconf, logger=self.logger)
        vad.compute_vad()
        self.print_info(u"Executing VAD... done")

        gf.delete_file(tmp_handler, tmp_file_path)

        if mode == u"speech":
            intervals = vad.speech
        elif mode == u"nonspeech":
            intervals = vad.nonspeech
        elif mode == u"both":
            speech = [[x[0], x[1], u"speech"] for x in vad.speech]
            nonspeech = [[x[0], x[1], u"nonspeech"] for x in vad.nonspeech]
            intervals = sorted(speech + nonspeech)
        intervals = [tuple(interval) for interval in intervals]
        self.write_to_file(output_file_path, intervals)

        return self.NO_ERROR_EXIT_CODE

    def write_to_file(self, output_file_path, intervals):
        """
        Write intervals to file.

        :param output_file_path: path of the output file to be written;
                                 if ``None``, print to stdout
        :type  output_file_path: string (path)
        :param intervals: a list of tuples, each representing an interval
        :type  intervals: list of tuples
        """
        msg = []
        if len(intervals) > 0:
            if len(intervals[0]) == 2:
                template = u"%.3f\t%.3f"
            else:
                template = u"%.3f\t%.3f\t%s"
            msg = [template % (interval) for interval in intervals]
        if output_file_path is None:
            self.print_info(u"Intervals detected:")
            for line in msg:
                self.print_generic(line)
        else:
            with io.open(output_file_path, "w", encoding="utf-8") as output_file:
                output_file.write(u"\n".join(msg))
                self.print_info(u"Created file %s" % output_file_path)



def main():
    """
    Execute program.
    """
    RunVADCLI().run(arguments=sys.argv)

if __name__ == '__main__':
    main()






#!/usr/bin/env python
# coding=utf-8

"""
**aeneas.tools** is a collection of command line tools
that can be run by the end user to perform tasks
such as computing or converting sync maps.
"""

__author__ = "Alberto Pettarin"
__copyright__ = """
    Copyright 2012-2013, Alberto Pettarin (www.albertopettarin.it)
    Copyright 2013-2015, ReadBeyond Srl   (www.readbeyond.it)
    Copyright 2015-2016, Alberto Pettarin (www.albertopettarin.it)
    """
__license__ = "GNU AGPL 3"
__version__ = "1.4.1"
__email__ = "aeneas@readbeyond.it"
__status__ = "Production"







#!/usr/bin/env python
# coding=utf-8

"""
Global constants, mostly default values,
public parameter names, and executable paths.
"""

__author__ = "Alberto Pettarin"
__copyright__ = """
    Copyright 2012-2013, Alberto Pettarin (www.albertopettarin.it)
    Copyright 2013-2015, ReadBeyond Srl   (www.readbeyond.it)
    Copyright 2015-2016, Alberto Pettarin (www.albertopettarin.it)
    """
__license__ = "GNU AGPL v3"
__version__ = "1.4.1"
__email__ = "aeneas@readbeyond.it"
__status__ = "Production"


### CONSTANTS ###

CONFIG_TXT_FILE_NAME = "config.txt"
""" File name for the TXT configuration file in containers """

CONFIG_XML_FILE_NAME = "config.xml"
""" File name for the XML configuration file in containers """

CONFIG_XML_TASKS_TAG = "tasks"
""" ``<tasks>`` tag in the XML configuration file """

CONFIG_XML_TASK_TAG = "task"
""" ``<task>`` tag in the XML configuration file """

CONFIG_RESERVED_CHARACTERS = ["~"]
""" List of reserved characters which are forbidden in configuration files """

CONFIG_STRING_SEPARATOR_SYMBOL = "|"
""" Separator of ``key=value`` pairs in config strings """

CONFIG_STRING_ASSIGNMENT_SYMBOL = "="
""" Assignment symbol in config string ``key=value`` pairs """

PARSED_TEXT_SEPARATOR = "|"
""" Separator for input text files in parsed format """

# reserved parameter names (RPN)
RPN_JOB_IDENTIFIER = "job_identifier"
"""
The identifier of a job. Reserved.

Usage: reserved
"""

RPN_TASK_IDENTIFIER = "task_identifier"
"""
The identifier of a task. Reserved.

Usage: reserved
"""

# public parameter names (PPN)
PPN_JOB_DESCRIPTION = "job_description"
"""
A human-readable description of the job.

Usage: config string, TXT config file, XML config file

Values: string

Example::

    job_description=This is a sample description

"""

PPN_JOB_LANGUAGE = "job_language"
"""
The language of the job.

Usage: config string, TXT config file, XML config file

Values: listed in :class:`aeneas.language.Language`

Example::

    job_language=en

"""

PPN_JOB_IS_AUDIO_FILE_NAME_REGEX = "is_audio_file_name_regex"
"""
The regex to match audio files in this job.

Usage: config string, TXT config file

Values: regex

Example::

    is_audio_file_name_regex=.*\.mp3
    is_audio_file_name_regex=audio.ogg

"""

PPN_JOB_IS_AUDIO_FILE_RELATIVE_PATH = "is_audio_file_relative_path"
"""
The path, relative to the task root directory,
where the audio files should be searched in input containers.

Usage: config string, TXT config file

Values: string (path)

Example::

    is_audio_file_relative_path=../audio
    is_audio_file_relative_path=mp3
    is_audio_file_relative_path=.

"""

PPN_JOB_IS_HIERARCHY_PREFIX = "is_hierarchy_prefix"
"""
The path, relative to the position of the TXT/XML config file,
to be considered the task root directory, in input containers.

Usage: config string, TXT config file

Values: string (path)

Example::

    is_hierarchy_prefix=OEBPS/Resources
    is_hierarchy_prefix=.

"""

PPN_JOB_IS_HIERARCHY_TYPE = "is_hierarchy_type"
"""
The type of hierarchy of the input job container.

Usage: config string, TXT config file

Values: listed in :class:`aeneas.hierarchytype.HierarchyType`

Example::

    is_hierarchy_type=flat
    is_hierarchy_type=paged

"""

PPN_JOB_IS_TASK_DIRECTORY_NAME_REGEX = "is_task_dir_name_regex"
"""
The regex to match task directory names
within the base task directory in input containers.
Applies to paged hierarchies only.

Usage: config string, TXT config file

Values: regex

Example::

    is_task_dir_name_regex=[0-9]+
    is_text_dir_name_regex=page[0-9]+

"""

PPN_JOB_IS_TEXT_FILE_FORMAT = "is_text_type"
"""
The text file format of text files in input containers.

Usage: config string, TXT config file, XML config file

Values: listed in :class:`aeneas.textfile.TextFileFormat`

Example::

    is_text_type=plain
    is_text_type=parsed
    is_text_type=unparsed

"""

PPN_JOB_IS_TEXT_FILE_NAME_REGEX = "is_text_file_name_regex"
"""
The regex for matching the text file name
of tasks in input containers.

Usage: config string, TXT config file

Values: regex

Example::

    is_text_file_name_regex=.*\.xhtml
    is_text_file_name_regex=page.xhtml

"""

PPN_JOB_IS_TEXT_FILE_RELATIVE_PATH = "is_text_file_relative_path"
"""
The path, relative to the task root directory,
where the text files should be searched in input containers.

Usage: config string, TXT config file

Values: string (path)

Example::

    is_audio_file_relative_path=../pages
    is_audio_file_relative_path=xhtml
    is_audio_file_relative_path=.

"""

PPN_JOB_IS_TEXT_UNPARSED_CLASS_REGEX = "is_text_unparsed_class_regex"
"""
The regex for matching the ``class`` attribute
of XML elements containing text fragments to be extracted
from ``unparsed`` text files.

Usage: config string, TXT config file, XML config file

Values: regex

Example::

    is_text_unparsed_class_regex=ra
    is_text_unparsed_class_regex=readaloud
    is_text_unparsed_class_regex=ra[0-9]+

"""

PPN_JOB_IS_TEXT_UNPARSED_ID_REGEX = "is_text_unparsed_id_regex"
"""
The regex for matching the ``id`` attribute
of XML elements containing text fragments to be extracted
from ``unparsed`` text files.

Usage: config string, TXT config file, XML config file

Values: regex

Example::

    is_text_unparsed_id_regex=f[0-9]+
    is_text_unparsed_id_regex=ra.*

"""

PPN_JOB_IS_TEXT_UNPARSED_ID_SORT = "is_text_unparsed_id_sort"
"""
The sorting algorithm to be used to sort the text fragments
extracted from ``unparsed`` text files, based on their ``id`` attributes.

Usage: config string, TXT config file, XML config file

Values: listed in :class:`aeneas.idsortingalgorithm.IDSortingAlgorithm`

Example::

    is_text_unparsed_id_sort=lexicographic
    is_text_unparsed_id_sort=numeric
    is_text_unparsed_id_sort=unsorted

"""

PPN_JOB_OS_CONTAINER_FORMAT = "os_job_file_container"
"""
The format of the output container.

Usage: config string, TXT config file, XML config file

Values: listed in :class:`aeneas.container.ContainerFormat`

Example::

    os_job_file_container=zip

"""

PPN_JOB_OS_FILE_NAME = "os_job_file_name"
"""
The file name of the output container.

Usage: config string, TXT config file, XML config file

Values: string

Example::

    os_job_file_name=output_sync_maps.zip

"""

PPN_JOB_OS_HIERARCHY_PREFIX = "os_job_file_hierarchy_prefix"
"""
The path of the root directory of the output container,
under which the task directories will be created.

Usage: config string, TXT config file, XML config file

Values: string (path)

Example::

    os_job_file_hierarchy_prefix=OEBPS/Resources

"""

PPN_JOB_OS_HIERARCHY_TYPE = "os_job_file_hierarchy_type"
"""
The type of output container structure.

Usage: config string, TXT config file, XML config file

Values: listed in :class:`aeneas.hierarchytype.HierarchyType`

Example::

    os_job_file_hierarchy_type=flat
    os_job_file_hierarchy_type=paged

"""

PPN_SYNCMAP_LANGUAGE = "language"
"""
Key for specifying the syncmap language

Values: listed in :class:`aeneas.language.Language`

Example::

    language=en
    language=it

.. versionadded:: 1.2.0
"""

PPN_TASK_CUSTOM_ID = "task_custom_id"
"""
The custom, human-readable identifier of a task.

Usage: config string, XML config file

Values: string

Example::

    task_custom_id=sonnet001

"""

PPN_TASK_DESCRIPTION = "task_description"
"""
The description of a task.

Usage: config string, XML config file

Values: string

Example::

    task_description=This is a sample description

"""

PPN_TASK_LANGUAGE = "task_language"
"""
The language of a task.

Usage: config string, XML config file

Values: listed in :class:`aeneas.language.Language`

Example::

    task_language=en

"""

PPN_TASK_ADJUST_BOUNDARY_ALGORITHM = "task_adjust_boundary_algorithm"
"""
The algorithm to be run to adjust the fragment boundaries.
If ``None`` or ``auto``, keep the current boundaries.

Usage: config string, TXT config file, XML config file

Values: listed in :class:`aeneas.adjustboundaryalgorithm.AdjustBoundaryAlgorithm`

Example::

    task_adjust_boundary_algorithm=aftercurrent
    task_adjust_boundary_algorithm=auto
    task_adjust_boundary_algorithm=beforenext
    task_adjust_boundary_algorithm=offset
    task_adjust_boundary_algorithm=percent
    task_adjust_boundary_algorithm=rate
    task_adjust_boundary_algorithm=rateaggressive

.. versionadded:: 1.0.4
"""

PPN_TASK_ADJUST_BOUNDARY_AFTERCURRENT_VALUE = "task_adjust_boundary_aftercurrent_value"
"""
The new boundary between two consecutive fragments
will be set at ``value`` seconds
after the end of the first fragment.

Requires ``task_adjust_boundary_algorithm=aftercurrent``.

Usage: config string, TXT config file, XML config file

Values: float

Example::

    task_adjust_boundary_aftercurrent_value=0.150

.. versionadded:: 1.0.4
"""

PPN_TASK_ADJUST_BOUNDARY_BEFORENEXT_VALUE = "task_adjust_boundary_beforenext_value"
"""
The new boundary between two consecutive fragments
will be set at ``value`` seconds
before the beginning of the second fragment.

Requires ``task_adjust_boundary_algorithm=beforenext``.

Usage: config string, TXT config file, XML config file

Values: float

Example::

    task_adjust_boundary_beforenext_value=0.200

.. versionadded:: 1.0.4
"""

PPN_TASK_ADJUST_BOUNDARY_OFFSET_VALUE = "task_adjust_boundary_offset_value"
"""
The new boundary between two consecutive fragments
will be set at ``value`` seconds from the current value.
A negative ``value`` will move the boundary back,
a positive ``value`` will move the boundary forward.

Requires ``task_adjust_boundary_algorithm=offset``.

Usage: config string, TXT config file, XML config file

Values: float

Example::

    task_adjust_boundary_offset_value=-0.200
    task_adjust_boundary_offset_value=0.150

.. versionadded:: 1.1.0
"""

PPN_TASK_ADJUST_BOUNDARY_PERCENT_VALUE = "task_adjust_boundary_percent_value"
"""
The new boundary between two consecutive fragments
will be set at this ``value`` percent
of the nonspeech interval between the two fragments.
The value must be between ``0`` and ``100``.

Requires ``task_adjust_boundary_algorithm=percent``.

Usage: config string, TXT config file, XML config file

Values: int

Example::

    task_adjust_boundary_percent_value=0
    task_adjust_boundary_percent_value=50
    task_adjust_boundary_percent_value=75
    task_adjust_boundary_percent_value=100

.. versionadded:: 1.0.4
"""

PPN_TASK_ADJUST_BOUNDARY_RATE_VALUE = "task_adjust_boundary_rate_value"
"""
The new boundary will be set trying to keep the rate
of all the fragments below this ``value`` characters/second.
The value must be greater than ``0``.

Requires ``task_adjust_boundary_algorithm=rate`` or
``task_adjust_boundary_algorithm=rateaggressive``.

Usage: config string, TXT config file, XML config file

Values: float

Example::

    task_adjust_boundary_rate_value=21.0

.. versionadded:: 1.0.4
"""

PPN_TASK_IS_AUDIO_FILE_DETECT_HEAD_MAX = "is_audio_file_detect_head_max"
"""
When synchronizing, auto detect the head of the audio file,
using the provided value as an upper bound, and disregard
these many seconds from the beginning of the audio file.

If the ``is_audio_file_head_length`` parameter is also provided,
the auto detection will not take place.

NOTE: This is an experimental feature, use with caution.

Usage: config string, XML config file

Values: float

Example::

    is_audio_file_detect_head_max=10.0

.. versionadded:: 1.2.0
"""

PPN_TASK_IS_AUDIO_FILE_DETECT_HEAD_MIN = "is_audio_file_detect_head_min"
"""
When synchronizing, auto detect the head of the audio file,
using the provided value as a lower bound, and disregard
these many seconds from the beginning of the audio file.

If the ``is_audio_file_head_length`` parameter is also provided,
the auto detection will not take place.

NOTE: This is an experimental feature, use with caution.

Usage: config string, XML config file

Values: float

Example::

    is_audio_file_detect_head_min=3.0

.. versionadded:: 1.2.0
"""

PPN_TASK_IS_AUDIO_FILE_DETECT_TAIL_MAX = "is_audio_file_detect_tail_max"
"""
When synchronizing, auto detect the tail of the audio file,
using the provided value as an upper bound, and disregard
these many seconds from the end of the audio file.

If the ``is_audio_file_process_length`` parameter or
the ``is_audio_file_tail_length`` parameter
are also provided,
the auto detection will not take place.

NOTE: This is an experimental feature, use with caution.

Usage: config string, XML config file

Values: float

Example::

    is_audio_file_detect_tail_max=10.0

.. versionadded:: 1.2.0
"""

PPN_TASK_IS_AUDIO_FILE_DETECT_TAIL_MIN = "is_audio_file_detect_tail_min"
"""
When synchronizing, auto detect the tail of the audio file,
using the provided value as a lower bound, and disregard
these many seconds from the end of the audio file.

If the ``is_audio_file_process_length`` parameter or
the ``is_audio_file_tail_length`` parameter
are also provided,
the auto detection will not take place.

NOTE: This is an experimental feature, use with caution.

Usage: config string, XML config file

Values: float

Example::

    is_audio_file_detect_tail_min=0.0

.. versionadded:: 1.2.0
"""

PPN_TASK_IS_AUDIO_FILE_HEAD_LENGTH = "is_audio_file_head_length"
"""
When synchronizing, disregard
these many seconds from the beginning of the audio file.

Usage: config string, XML config file

Values: float

Example::

    is_audio_file_head_length=12.345

"""

PPN_TASK_IS_AUDIO_FILE_PROCESS_LENGTH = "is_audio_file_process_length"
"""
When synchronizing, process only these many seconds
from the audio file, starting at the beginning of the file
or at the end of the head.

Usage: config string, XML config file

Values: float

Example::

    is_audio_file_process_length=987.654

"""

PPN_TASK_IS_AUDIO_FILE_TAIL_LENGTH = "is_audio_file_tail_length"
"""
When synchronizing, disregard
these many seconds from the end of the audio file.

Note that if both ``is_audio_file_process_length``
and ``is_audio_file_tail_length`` are provided,
only the former will be taken into account,
and ``is_audio_file_tail_length`` will be ignored.

Usage: config string, XML config file

Values: float

Example::

    is_audio_file_tail_length=12.345

"""

PPN_TASK_IS_TEXT_FILE_FORMAT = "is_text_type"
"""
The format of the input text file.

Usage: config string, TXT config file, XML config file

Values: listed in :class:`aeneas.textfile.TextFileFormat`

Example::

    is_text_type=plain
    is_text_type=parsed
    is_text_type=unparsed

"""

PPN_TASK_IS_TEXT_FILE_IGNORE_REGEX = "is_text_file_ignore_regex"
"""
The regex to match text to be ignored for alignment purposes.
The output sync map file will contain the original text.

Usage: config string, TXT config file, XML config file

Values: regex

Example::

    is_text_file_ignore_regex=\\[.*?\\]

"""

PPN_TASK_IS_TEXT_FILE_TRANSLITERATE_MAP = "is_text_file_transliterate_map"
"""
The path to the transliteration map file to be used to delete/replace
characters in the input text file for alignment purposes.
The output sync map file will contain the original text.

Usage: config string, TXT config file, XML config file

Values: string (path)

Example::

    is_text_file_transliterate_map=trans.map

"""

PPN_TASK_IS_TEXT_UNPARSED_CLASS_REGEX = "is_text_unparsed_class_regex"
"""
The regex to match ``class`` attributes for text fragments.
It applies to ``unparsed`` text files only.

Usage: config string, TXT config file, XML config file

Values: regex

Example::

    is_text_unparsed_class_regex=ra
    is_text_unparsed_class_regex=readaloud
    is_text_unparsed_class_regex=ra[0-9]+

"""

PPN_TASK_IS_TEXT_UNPARSED_ID_REGEX = "is_text_unparsed_id_regex"
"""
The regex to match ``id`` attributes for text fragments.
It applies to ``unparsed`` text files only.

Usage: config string, TXT config file, XML config file

Values: regex

Example::

    is_text_unparsed_id_regex=f[0-9]+
    is_text_unparsed_id_regex=ra.*

"""

PPN_TASK_IS_TEXT_UNPARSED_ID_SORT = "is_text_unparsed_id_sort"
"""
The algorithm to sort text fragments by their ``id`` attributes.
It applies to unparsed text files only.

Usage: config string, TXT config file, XML config file

Values: listed in :class:`aeneas.idsortingalgorithm.IDSortingAlgorithm`

Example::

    is_text_unparsed_id_sort=lexicographic
    is_text_unparsed_id_sort=numeric
    is_text_unparsed_id_sort=unsorted

"""

PPN_TASK_OS_FILE_FORMAT = "os_task_file_format"
"""
The format of the sync map output for the task.

Usage: config string, TXT config file, XML config file

Values: listed in :class:`aeneas.syncmap.SyncMapFormat`

Example::

    os_task_file_format=smil
    os_task_file_format=txt
    os_task_file_format=srt

"""

PPN_TASK_OS_FILE_ID_REGEX = "os_task_file_id_regex"
"""
The regex to be used for the fragment identifiers
of the sync map output file.

This parameter will be used only
when the input text file has `plain` or `subtitles` format;
for `parsed` and `unparsed` input text files, the identifiers
contained in the input text file will be used instead.

When specified, the value must contain an interger placeholder,
for example ``%d`` or ``%06d``.

Usage: config string, TXT config file, XML config file

Values: string

Example::

    os_task_file_id_regex=f%06d
    os_task_file_id_regex=Word%03d

.. versionadded:: 1.3.1
"""

PPN_TASK_OS_FILE_NAME = "os_task_file_name"
"""
The name of the sync map file output for the task.

If processing a Job,
the value might contain the ``PPV_OS_TASK_PREFIX`` placeholder,
that will be replaced by a suitable path string.

Usage: config string, TXT config file, XML config file

Values: string

Example::

    os_task_file_name=map.smil

"""

PPN_TASK_OS_FILE_SMIL_AUDIO_REF = "os_task_file_smil_audio_ref"
"""
The value of the ``src`` attribute for the ``<audio>`` element
in the output sync map.
It applies to ``SMIL`` sync maps only.

Usage: config string, TXT config file, XML config file

Values: string

Example::

    os_task_file_smil_audio_ref=../audio/p001.mp3
    os_task_file_smil_audio_ref=audio.mp3

"""

PPN_TASK_OS_FILE_SMIL_PAGE_REF = "os_task_file_smil_page_ref"
"""
The value of the ``src`` attribute for the ``<text>`` element
in the output sync map.
It applies to ``SMIL`` sync maps only.

Usage: config string, TXT config file, XML config file

Values: string

Example::

    os_task_file_smil_page_ref=../xhtml/page.xhtml
    os_task_file_smil_page_ref=p001.xhtml

"""

PPN_TASK_OS_FILE_HEAD_TAIL_FORMAT = "os_task_file_head_tail_format"
"""
The format of the head and tail of the sync map output for the task.

Usage: config string, TXT config file, XML config file

Values: listed in :class:`aeneas.syncmap.SyncMapHeadTailFormat`

Example::

    os_task_file_head_tail_format=add
    os_task_file_head_tail_format=hidden
    os_task_file_head_tail_format=stretch

.. versionadded:: 1.2.0
"""

PPN_TASK_IS_TEXT_FILE_XML = "is_text_file"
"""
Key for the path, relative to the XML config file,
of the text file of the current task

Usage: XML config file

Values: string (path)

Example::

    <is_text_file>OEBPS/Resources/sonnet001.txt</is_text_file>

"""

PPN_TASK_IS_AUDIO_FILE_XML = "is_audio_file"
"""
Key for the path, relative to the XML config file,
of the audio file of the current task

Usage: XML config file

Values: string (path)

Example::

    <is_audio_file>OEBPS/Resources/sonnet001.mp3</is_audio_file>

"""

# public parameter values (PPV)
PPV_OS_TASK_PREFIX = "$PREFIX"
"""
Placeholder for the actual task directory or task_custom_id value.

Usage: TXT config file

Example::

    os_task_file_name=$PREFIX.smil

"""



### RUNTIMECONFIGURATION ###

RC_ALLOW_UNLISTED_LANGUAGES = "allow_unlisted_languages"
""" If ``True``, allow using a language code not listed in ``languages.py``;
otherwise, raise an error if the user attempts to use a language not listed.
Default: ``True``. """

RC_C_EXTENSIONS = "c_extensions"
""" If ``True`` and Python C extensions are available, use them.
Otherwise, use pure Python code.
Default: ``True``. """

RC_DTW_ALGORITM = "dtw_algorithm"
""" DTW aligner algorithm.
Default: ``stripe``. """

RC_DTW_MARGIN = "dtw_margin"
""" DTW aligner margin, in seconds, for the ``stripe`` algorithm.
Default: ``60``, corresponding to ``60s`` ahead and behind
(i.e., ``120s`` total margin). """

RC_ESPEAK_PATH = "espeak_path"
""" Path to the ``espeak`` executable.
Default: ``espeak``. """

RC_FFMPEG_PATH = "ffmpeg_path"
""" Path to the ``ffmpeg`` executable.
Default: ``ffmpeg``. """

RC_FFMPEG_SAMPLE_RATE = "ffmpeg_sample_rate"
""" Sample rate for ``ffmpeg``, in Hertz.
Default: ``16000``. """

RC_FFPROBE_PATH = "ffprobe_path"
""" Path to the ``ffprobe`` executable.
Default: ``ffprobe``. """

RC_JOB_MAX_TASKS = "job_max_tasks"
""" Maximum number of Tasks of a Job.
If a Job has more Tasks than this value,
it will not be executed and an error will be raised.
Use ``0`` for disabling this check.
Default: ``0`` (disabled). """

RC_MFCC_FILTERS = "mfcc_filters"
""" Number of filters for extracting MFCCs.
Default: ``40``. """

RC_MFCC_SIZE = "mfcc_size"
""" Number of MFCCs to extract, including the 0th.
Default: ``13``. """

RC_MFCC_FFT_ORDER = "mfcc_fft_order"
""" Order of the RFFT for extracting MFCCs.
It must be a power of two.
Default: ``512``. """

RC_MFCC_LOWER_FREQUENCY = "mfcc_lower_frequency"
""" Lower frequency to be used for extracting MFCCs, in Hertz.
Default: ``133.3333``. """

RC_MFCC_UPPER_FREQUENCY = "mfcc_upper_frequency"
""" Upper frequency to be used for extracting MFCCs, in Hertz.
Default: ``6855.4976``. """

RC_MFCC_EMPHASIS_FACTOR = "mfcc_emphasis_factor"
""" Emphasis factor to be applied to MFCCs.
Default: ``0.970``. """

RC_MFCC_WINDOW_LENGTH = "mfcc_window_length"
""" Length of the window for extracting MFCCs, in seconds.
It is usual to set it between 1.5 and 4 times
the value of ``RC_MFCC_WINDOW_SHIFT``.
Default: ``0.100``. """

RC_MFCC_WINDOW_SHIFT = "mfcc_window_shift"
""" Shift of the window for extracting MFCCs, in seconds.
This parameter is basically the time step
of the synchronization maps output.
Default: ``0.040``. """

RC_TASK_MAX_AUDIO_LENGTH = "task_max_audio_length"
""" Maximum length of the audio file of a Task, in seconds.
If a Task has an audio file longer than this value,
it will not be executed and an error will be raised.
Use ``0`` for disabling this check.
Default: ``7200`` seconds. """

RC_TASK_MAX_TEXT_LENGTH = "task_max_text_length"
""" Maximum number of text fragments in the text file of a Task.
If a Task has more text fragments than this value,
it will not be executed and an error will be raised.
Use ``0`` for disabling this check.
Default: ``0`` (disabled). """

RC_TMP_PATH = "tmp_path"
""" Path to the temporary directory to be used.
Default: ``None``, meaning that the default temporary directory
will be set by ``RC_TMP_PATH_DEFAULT_POSIX``
or ``RC_TMP_PATH_DEFAULT_NONPOSIX``. """

RC_VAD_EXTEND_SPEECH_INTERVAL_AFTER = "vad_extend_speech_after"
"""
Extend to the right (after/future)
a speech interval found by the VAD algorithm,
by this many seconds.
Default: ``0`` seconds.

.. versionadded:: 1.0.4
"""

RC_VAD_EXTEND_SPEECH_INTERVAL_BEFORE = "vad_extend_speech_before"
"""
Extend to the left (before/past)
a speech interval found by the VAD algorithm,
by this many seconds.
Default: ``0`` seconds.

.. versionadded:: 1.0.4
"""

RC_VAD_LOG_ENERGY_THRESHOLD = "vad_log_energy_threshold"
"""
Threshold for the VAD algorithm to decide
that a given frame contains speech.
Note that this is the log10 of the energy coefficient.
Default: ``0.699`` = ``log10(5)``, that is, a frame must have
an energy at least 5 times higher than the minimum
to be considered a speech frame.

.. versionadded:: 1.0.4
"""

RC_VAD_MIN_NONSPEECH_LENGTH = "vad_min_nonspeech_length"
"""
Minimum length, in seconds, of a nonspeech interval.
Default: ``0.200`` seconds.

.. versionadded:: 1.0.4
"""



### DEFAULT VALUES ###

RC_TMP_PATH_DEFAULT_POSIX = "/tmp/"
""" Default temporary directory path for POSIX OSes. """

RC_TMP_PATH_DEFAULT_NONPOSIX = None
""" Default temporary directory path for non-POSIX OSes.
Set to ``None`` so that ``tempfile`` will select
the most approriate temporary directory root path. """






#!/usr/bin/env python
# coding=utf-8

"""
Check whether the setup of aeneas was successful.

Running the checks in this class makes sense only
if you git-cloned the original GitHub repository
and/or if you are interested in contributing to the
development of aeneas.
"""

from __future__ import absolute_import
from __future__ import print_function
import sys

import aeneas.globalfunctions as gf

__author__ = "Alberto Pettarin"
__copyright__ = """
    Copyright 2012-2013, Alberto Pettarin (www.albertopettarin.it)
    Copyright 2013-2015, ReadBeyond Srl   (www.readbeyond.it)
    Copyright 2015-2016, Alberto Pettarin (www.albertopettarin.it)
    """
__license__ = "GNU AGPL v3"
__version__ = "1.4.1"
__email__ = "aeneas@readbeyond.it"
__status__ = "Production"

SETUP_COMMAND = u"'python setup.py build_ext --inplace'"

ANSI_ERROR = u"\033[91m"
ANSI_OK = u"\033[92m"
ANSI_WARNING = u"\033[93m"
ANSI_END = u"\033[0m"

def print_error(msg):
    if gf.is_posix():
        print(u"%s[ERRO] %s%s" % (ANSI_ERROR, msg, ANSI_END))
    else:
        print(u"[ERRO] %s" % (msg))

def print_info(msg):
    print(u"[INFO] %s" % (msg))

def print_success(msg):
    if gf.is_posix():
        print(u"%s[INFO] %s%s" % (ANSI_OK, msg, ANSI_END))
    else:
        print(u"[INFO] %s" % (msg))

def print_warning(msg):
    if gf.is_posix():
        print(u"%s[WARN] %s%s" % (ANSI_WARNING, msg, ANSI_END))
    else:
        print(u"[WARN] %s" % (msg))

class Diagnostics(object):
    """
    Check whether the setup of aeneas was successful.
    """
    @classmethod
    def check_shell_encoding(cls):
        """
        Check whether the shell (sys.stdin and sys.stdout) is UTF-8 encoded.

        Return ``True`` on failure and ``False`` on success.

        :rtype: bool
        """
        is_in_utf8 = True
        is_out_utf8 = True
        if sys.stdin.encoding not in ["UTF-8", "UTF8"]:
            is_in_utf8 = False
        if sys.stdout.encoding not in ["UTF-8", "UTF8"]:
            is_out_utf8 = False
        if (is_in_utf8) and (is_out_utf8):
            print_success(u"shell encoding OK")
        else:
            print_warning(u"shell encoding WARNING")
            if not is_in_utf8:
                print_warning(u"  The default input encoding of your shell is not UTF-8")
            if not is_out_utf8:
                print_warning(u"  The default output encoding of your shell is not UTF-8")
            print_info(u"  If you plan to use aeneas on the command line,")
            if gf.is_posix():
                print_info(u"  you might want to 'export PYTHONIOENCODING=UTF-8' in your shell")
            else:
                print_info(u"  you might want to 'set PYTHONIOENCODING=UTF-8' in your shell")
            return True 
        return False

    @classmethod
    def check_ffprobe(cls):
        """
        Check whether ffprobe can be called.

        Return ``True`` on failure and ``False`` on success.

        :rtype: bool
        """
        try:
            from aeneas.ffprobewrapper import FFPROBEWrapper
            import aeneas.globalfunctions as gf
            file_path = gf.absolute_path(u"tools/res/audio.mp3", __file__)
            prober = FFPROBEWrapper()
            properties = prober.read_properties(file_path)
            print_success(u"ffprobe OK")
            return False
        except:
            pass
        print_error(u"ffprobe ERROR")
        print_info(u"  Please make sure you have ffprobe installed correctly")
        print_info(u"  (usually it is provided by the ffmpeg installer)")
        print_info(u"  and that its path is in your PATH environment variable")
        return True

    @classmethod
    def check_ffmpeg(cls):
        """
        Check whether ffmpeg can be called.

        Return ``True`` on failure and ``False`` on success.

        :rtype: bool
        """
        try:
            from aeneas.ffmpegwrapper import FFMPEGWrapper
            import aeneas.globalfunctions as gf
            input_file_path = gf.absolute_path(u"tools/res/audio.mp3", __file__)
            handler, output_file_path = gf.tmp_file(suffix=u".wav")
            converter = FFMPEGWrapper()
            result = converter.convert(input_file_path, output_file_path)
            gf.delete_file(handler, output_file_path)
            if result:
                print_success(u"ffmpeg OK")
                return False
        except:
            pass
        print_error(u"ffmpeg ERROR")
        print_info(u"  Please make sure you have ffmpeg installed correctly")
        print_info(u"  and that its path is in your PATH environment variable")
        return True

    @classmethod
    def check_espeak(cls):
        """
        Check whether espeak can be called.

        Return ``True`` on failure and ``False`` on success.

        :rtype: bool
        """
        try:
            from aeneas.espeakwrapper import ESPEAKWrapper
            from aeneas.language import Language
            import aeneas.globalfunctions as gf
            text = u"From fairest creatures we desire increase,"
            language = Language.EN
            handler, output_file_path = gf.tmp_file(suffix=u".wav")
            espeak = ESPEAKWrapper()
            result = espeak.synthesize_single(
                text,
                language,
                output_file_path
            )
            gf.delete_file(handler, output_file_path)
            if result:
                print_success(u"espeak OK")
                return False
        except:
            pass
        print_error(u"espeak ERROR")
        print_info(u"  Please make sure you have espeak installed correctly")
        print_info(u"  and that its path is in your PATH environment variable")
        print_info(u"  You might also want to check that the espeak-data directory")
        print_info(u"  is set up correctly, for example, it has the correct permissions")
        return True

    @classmethod
    def check_tools(cls):
        """
        Check whether aeneas.tools.* can be imported.

        Return ``True`` on failure and ``False`` on success.

        :rtype: bool
        """
        try:
            from aeneas.tools.convert_syncmap import ConvertSyncMapCLI
            from aeneas.tools.download import DownloadCLI
            from aeneas.tools.espeak_wrapper import ESPEAKWrapperCLI
            from aeneas.tools.execute_job import ExecuteJobCLI
            from aeneas.tools.execute_task import ExecuteTaskCLI
            from aeneas.tools.extract_mfcc import ExtractMFCCCLI
            from aeneas.tools.ffmpeg_wrapper import FFMPEGWrapperCLI
            from aeneas.tools.ffprobe_wrapper import FFPROBEWrapperCLI
            from aeneas.tools.read_audio import ReadAudioCLI
            from aeneas.tools.read_text import ReadTextCLI
            from aeneas.tools.run_sd import RunSDCLI
            from aeneas.tools.run_vad import RunVADCLI
            from aeneas.tools.synthesize_text import SynthesizeTextCLI
            from aeneas.tools.validate import ValidateCLI
            print_success(u"aeneas.tools OK")
            return False
        except:
            pass
        print_error(u"aeneas.tools ERROR")
        print_info(u"  Unable to import one or more aeneas.tools")
        print_info(u"  Please check that you installed aeneas properly")
        return True

    @classmethod
    def check_cdtw(cls):
        """
        Check whether Python C extension cdtw can be imported.

        Return ``True`` on failure and ``False`` on success.

        :rtype: bool
        """
        if gf.can_run_c_extension("cdtw"):
            print_success(u"aeneas.cdtw COMPILED")
            return False
        print_warning(u"aeneas.cdtw NOT COMPILED")
        print_info(u"  You can still run aeneas but it will be significantly slower")
        print_info(u"  To compile the cdtw module, run %s" % SETUP_COMMAND)
        return True

    @classmethod
    def check_cmfcc(cls):
        """
        Check whether Python C extension cmfcc can be imported.

        Return ``True`` on failure and ``False`` on success.

        :rtype: bool
        """
        if gf.can_run_c_extension("cmfcc"):
            print_success(u"aeneas.cmfcc COMPILED")
            return False
        print_warning(u"aeneas.cmfcc NOT COMPILED")
        print_info(u"  You can still run aeneas but it will be significantly slower")
        print_info(u"  To compile the cmfcc module, run %s" % SETUP_COMMAND)
        return True

    @classmethod
    def check_cew(cls):
        """
        Check whether Python C extension cew can be imported.

        Return ``True`` on failure and ``False`` on success.

        For those OSes where ``cew`` is not available,
        print a warning but also return ``False`` (success).

        :rtype: bool
        """
        if not gf.is_linux():
            print_warning(u"cew NOT AVAILABLE")
            print_info(u"  The Python C Extension cew is not available for your OS")
            print_info(u"  You can still run aeneas but it will be a bit slower (than Linux)")
            return False
        if gf.can_run_c_extension("cew"):
            print_success(u"aeneas.cew COMPILED")
            return False
        print_warning(u"aeneas.cew NOT COMPILED")
        print_info(u"  You can still run aeneas but it will be a bit slower")
        print_info(u"  To compile the cew module, run %s" % SETUP_COMMAND)
        return True

    @classmethod
    def check_all(cls, tools=True, encoding=True, c_ext=True):
        """
        Perform all checks.

        Return a tuple of booleans ``(errors, warnings, c_ext_warnings)``.

        :param tools: if ``True``, check aeneas tools
        :type  tools: bool
        :param encoding: if ``True``, check shell encoding
        :type  encoding: bool
        :param c_ext: if ``True``, check Python C extensions
        :type  c_ext: bool
        :rtype: (bool, bool, bool)
        """
        # errors are fatal
        if cls.check_ffprobe():
            return (True, False, False)
        if cls.check_ffmpeg():
            return (True, False, False)
        if cls.check_espeak():
            return (True, False, False)
        if (tools) and (cls.check_tools()):
            return (True, False, False)

        # warnings are non-fatal
        warnings = False
        c_ext_warnings = False

        if encoding:
            warnings = cls.check_shell_encoding()

        if c_ext:
            # we do not want lazy evaluation
            c_ext_warnings = cls.check_cdtw() or c_ext_warnings
            c_ext_warnings = cls.check_cmfcc() or c_ext_warnings
            c_ext_warnings = cls.check_cew() or c_ext_warnings

        return (False, warnings, c_ext_warnings)



def main():
    errors, warnings, c_ext_warnings = Diagnostics.check_all()
    if errors:
        sys.exit(1)
    #print_info(u"")
    if c_ext_warnings:
        print_warning(u"All required dependencies are met but at least one available Python C extension is not compiled")
        #print_info(u"You can still run aeneas but it will be slower")
        #print_info(u"Enjoy running aeneas!")
        sys.exit(2)
    else:
        print_success(u"All required dependencies are met and all available Python C extensions are compiled")
        #print_info(u"Enjoy running aeneas!")
        sys.exit(0)



if __name__ == '__main__':
    main()






#!/usr/bin/env python
# coding=utf-8

"""
Basically a dictionary with a fixed set of keys,
with default values and aliases.
"""

from __future__ import absolute_import
from __future__ import print_function

import aeneas.globalconstants as gc
import aeneas.globalfunctions as gf

__author__ = "Alberto Pettarin"
__copyright__ = """
    Copyright 2012-2013, Alberto Pettarin (www.albertopettarin.it)
    Copyright 2013-2015, ReadBeyond Srl   (www.readbeyond.it)
    Copyright 2015-2016, Alberto Pettarin (www.albertopettarin.it)
    """
__license__ = "GNU AGPL v3"
__version__ = "1.4.1"
__email__ = "aeneas@readbeyond.it"
__status__ = "Production"

class ConfigurationObject(object):
    """
    A structure representing a generic configuration object, that is,
    a dictionary with a fixed set of keys,
    each with a default value, type, and possibly aliases.

    Values are stored as Unicode strings, and casted to int or float
    when accessed.

    :param config_string: the job configuration string
    :type  config_string: Unicode string

    :raises TypeError: if ``config_string`` is not ``None`` and
                       it is not a Unicode string
    :raises KeyError: if trying to access a key not listed above
    """

    TAG = u"ConfigurationObject"

    FIELDS = [
        #
        # in subclasses, create fields like this:
        # (field_name, (default_value, conversion_function, [alias1, alias2, ...]))
        #
        # examples:
        # (gc.FOO, (None, None, ["foo"]))
        # (gc.BAR, (0.0, float, ["bar", "baz"]))
        #
    ]

    def __init__(self, config_string=None):
        if (config_string is not None) and (not gf.is_unicode(config_string)):
            raise TypeError("config_string is not a Unicode string")

        # set dictionaries up to keep the config data
        self.data = {}
        self.types = {}
        self.aliases = {}
        for (field, info) in self.FIELDS:
            (fdefault, ftype, faliases) = info
            self.data[field] = fdefault
            self.types[field] = ftype
            for alias in faliases:
                self.aliases[alias] = field

        if config_string is not None:
            # strip leading/trailing " or ' characters
            if (config_string[0] == config_string[-1]) and (config_string[0] in [u"\"", u"'"]):
                config_string = config_string[1:-1]
            # populate values from config_string,
            # ignoring keys not present in FIELDS
            properties = gf.config_string_to_dict(config_string)
            for key in set(properties.keys()) & set(self.data.keys()):
                self.data[key] = properties[key]

    def __contains__(self, key):
        return (key in self.data) or (key in self.aliases)

    def __setitem__(self, key, value):
        if key in self.aliases:
            key = self.aliases[key]
        if key in self.data:
            self.data[key] = value
        else:
            raise KeyError(key)

    def __getitem__(self, key):
        if key in self.aliases:
            key = self.aliases[key]
        if key in self.data:
            return self._cast(key, self.data[key])
        else:
            raise KeyError(key)

    def __unicode__(self):
        return u"\n".join([u"%s: '%s'" % (fn, self.data[fn]) for fn in sorted(self.data.keys())])

    def __str__(self):
        return gf.safe_str(self.__unicode__())

    def _cast(self, key, value):
        if (value is not None) and (self.types[key] is not None):
            if self.types[key] is bool:
                return value in [True, u"True", u"true", u"Yes", u"yes", u"1", 1]
            else:
                return self.types[key](value)
        return value

    def config_string(self):
        """
        Build the storable string corresponding
        to this job configuration object.

        :rtype: string
        """
        return (gc.CONFIG_STRING_SEPARATOR_SYMBOL).join(
            [u"%s%s%s" % (fn, gc.CONFIG_STRING_ASSIGNMENT_SYMBOL, self.data[fn]) for fn in sorted(self.data.keys()) if self.data[fn] is not None]
        )






#!/usr/bin/env python
# coding=utf-8

"""
A logger class to help with debugging and performance tests.
"""

from __future__ import absolute_import
from __future__ import print_function
import datetime
import io

from aeneas.runtimeconfiguration import RuntimeConfiguration
import aeneas.globalfunctions as gf

__author__ = "Alberto Pettarin"
__copyright__ = """
    Copyright 2012-2013, Alberto Pettarin (www.albertopettarin.it)
    Copyright 2013-2015, ReadBeyond Srl   (www.readbeyond.it)
    Copyright 2015-2016, Alberto Pettarin (www.albertopettarin.it)
    """
__license__ = "GNU AGPL v3"
__version__ = "1.4.1"
__email__ = "aeneas@readbeyond.it"
__status__ = "Production"

class Logger(object):
    """
    A logger class to help with debugging and performance tests.

    :param tee: if ``True``, tee (i.e., log and print to stdout)
    :type  tee: bool
    :param indentation: the initial indentation of the log
    :type  indentation: int
    :param tee_show_datetime: if ``True``, print date and time when teeing
    :type  tee_show_datetime: bool
    :param rconf: a runtime configuration. Default: ``None``, meaning that
                  default settings will be used.
    :type  rconf: :class:`aeneas.runtimeconfiguration.RuntimeConfiguration`
    """

    DEBUG = "DEBU"
    """ ``DEBUG`` severity """

    INFO = "INFO"
    """ ``INFO`` severity """

    WARNING = "WARN"
    """ ``WARNING`` severity """

    CRITICAL = "CRIT"
    """ ``CRITICAL`` severity """

    def __init__(self, tee=False, indentation=0, tee_show_datetime=True, rconf=None):
        self.entries = []
        self.tee = tee
        self.indentation = indentation
        self.tee_show_datetime = tee_show_datetime
        self.rconf = rconf or RuntimeConfiguration()

    def __len__(self):
        return len(self.entries)

    def __unicode__(self):
        return self.pretty_print()

    def __str__(self):
        return gf.safe_str(self.__unicode__())

    def __repr__(self):
        return u"Logger(tee=%s, indentation=%d, tee_show_datetime=%s)" % (
            self.tee,
            self.indentation,
            self.tee_show_datetime
        )

    @property
    def tee(self):
        """
        If ``True``, tee (i.e., log and print to stdout).

        :rtype: bool
        """
        return self.__tee
    @tee.setter
    def tee(self, tee):
        self.__tee = tee

    @property
    def tee_show_datetime(self):
        """
        If ``True``, print date and time when teeing.

        :rtype: bool
        """
        return self.__tee_show_datetime
    @tee_show_datetime.setter
    def tee_show_datetime(self, tee_show_datetime):
        self.__tee_show_datetime = tee_show_datetime

    @property
    def indentation(self):
        """
        The current indentation of the log.
        Useful to visually distinguish log levels.

        :rtype: int
        """
        return self.__indentation
    @indentation.setter
    def indentation(self, indentation):
        self.__indentation = indentation

    def pretty_print(self, as_list=False, show_datetime=True):
        """
        Return a Unicode string pretty print of the log entries.

        :param as_list: if ``True``, return a list of Unicode strings,
                        one for each entry, instead of a Unicode string
        :type  as_list: bool
        :param show_datetime: if ``True``, show the date and time of the entries
        :type  show_datetime: bool
        :rtype: Unicode string or list of Unicode strings
        """
        ppl = [entry.pretty_print(show_datetime) for entry in self.entries]
        if as_list:
            return ppl
        return u"\n".join(ppl)

    def log(self, message, severity=INFO, tag=u""):
        """
        Add a given message to the log.

        :param message: the message to be added
        :type  message: Unicode string
        :param severity: the severity of the message
        :type  severity: :class:`aeneas.logger.Logger` enum
        :param tag: the tag associated with the message;
                    usually, the name of the class generating the entry
        :type  tag: Unicode string
        """
        entry = _LogEntry(
            severity=severity,
            time=datetime.datetime.now(),
            tag=tag,
            indentation=self.indentation,
            message=self._sanitize(message)
        )
        self.entries.append(entry)
        if self.tee:
            gf.safe_print(entry.pretty_print(show_datetime=self.tee_show_datetime))

    def clear(self):
        """
        Clear the contents of the log.
        """
        self.entries = []

    def write(self, path):
        """
        Output the log to file.

        :param path: the path of the log file to be written
        :type  path: Unicode string (path)
        """
        with io.open(path, "w", encoding="utf-8") as log_file:
            log_file.write(self.pretty_print())

    @classmethod
    def _sanitize(cls, message):
        """
        Sanitize the given message,
        dealing with multiple arguments
        and/or string formatting.

        :param message: the log message to be sanitized
        :type  message: Unicode string or list of Unicode strings
        :rtype: Unicode string
        """
        if isinstance(message, list):
            if len(message) == 0:
                sanitized = u"Empty log message"
            elif len(message) == 1:
                sanitized = message[0]
            else:
                sanitized = message[0] % tuple(message[1:])
        else:
            sanitized = message
        if not gf.is_unicode(sanitized):
            raise TypeError("The given log message is not a Unicode string")
        return sanitized



class _LogEntry(object):
    """
    A structure for a log entry.
    """

    def __init__(self, message, severity, tag, indentation, time):
        self.message = message
        self.severity = severity
        self.tag = tag
        self.indentation = indentation
        self.time = time

    def pretty_print(self, show_datetime=True):
        """
        Returns a Unicode string containing
        the pretty printing of a given log entry.

        :param show_datetime: if ``True``, print the date and time of the entry
        :type  show_datetime: bool
        :rtype: Unicode string
        """
        if show_datetime:
            return u"[%s] %s %s%s: %s" % (
                self.severity,
                gf.object_to_unicode(self.time),
                u" " * self.indentation,
                self.tag,
                self.message
            )
        return u"[%s] %s%s: %s" % (
            self.severity,
            u" " * self.indentation,
            self.tag,
            self.message
        )

    @property
    def message(self):
        """
        The message of this log entry.

        :rtype: Unicode string
        """
        return self.__message
    @message.setter
    def message(self, message):
        self.__message = message

    @property
    def severity(self):
        """
        The severity of this log entry.

        :rtype: :class:`aeneas.logger.Logger` enum
        """
        return self.__severity
    @severity.setter
    def severity(self, severity):
        self.__severity = severity

    @property
    def tag(self):
        """
        The tag of this log entry.

        :rtype: Unicode string
        """
        return self.__tag
    @tag.setter
    def tag(self, tag):
        self.__tag = tag

    @property
    def indentation(self):
        """
        The indentation of this log entry.

        :rtype: Unicode string
        """
        return self.__indentation
    @indentation.setter
    def indentation(self, indentation):
        self.__indentation = indentation

    @property
    def time(self):
        """
        The date and time of this log entry.

        :rtype: datetime.time
        """
        return self.__time
    @time.setter
    def time(self, time):
        self.__time = time






#!/usr/bin/env python
# coding=utf-8

"""
Detect the audio head and/or tail of the given audio file.
"""

from __future__ import absolute_import
from __future__ import print_function
import sys

from aeneas.audiofile import AudioFileMonoWAVE
from aeneas.ffmpegwrapper import FFMPEGWrapper
from aeneas.language import Language
from aeneas.sd import SD
from aeneas.tools.abstract_cli_program import AbstractCLIProgram
import aeneas.globalconstants as gc
import aeneas.globalfunctions as gf

__author__ = "Alberto Pettarin"
__copyright__ = """
    Copyright 2012-2013, Alberto Pettarin (www.albertopettarin.it)
    Copyright 2013-2015, ReadBeyond Srl   (www.readbeyond.it)
    Copyright 2015-2016, Alberto Pettarin (www.albertopettarin.it)
    """
__license__ = "GNU AGPL 3"
__version__ = "1.4.1"
__email__ = "aeneas@readbeyond.it"
__status__ = "Production"

class RunSDCLI(AbstractCLIProgram):
    """
    Detect the audio head and/or tail of the given audio file.
    """
    AUDIO_FILE = gf.relative_path("res/audio.mp3", __file__)
    PARAMETERS_HEAD = "--min-head=0.0 --max-head=5.0"
    PARAMETERS_TAIL = "--min-tail=1.0 --max-tail=5.0"
    TEXT_FILE = gf.relative_path("res/parsed.txt", __file__)

    NAME = gf.file_name_without_extension(__file__)

    HELP = {
        "description": u"Detect the audio head and/or tail of the given audio file.",
        "synopsis": [
            u"list 'fragment 1|fragment 2|...|fragment N' LANGUAGE AUDIO_FILE",
            u"[parsed|plain|subtitles|unparsed] TEXT_FILE LANGUAGE AUDIO_FILE"
        ],
        "examples": [
            u"parsed %s en %s" % (TEXT_FILE, AUDIO_FILE),
            u"parsed %s en %s %s" % (TEXT_FILE, AUDIO_FILE, PARAMETERS_HEAD),
            u"parsed %s en %s %s" % (TEXT_FILE, AUDIO_FILE, PARAMETERS_TAIL),
            u"parsed %s en %s %s %s" % (TEXT_FILE, AUDIO_FILE, PARAMETERS_HEAD, PARAMETERS_TAIL),
        ],
        "options": [
            u"--class-regex=REGEX : extract text from elements with class attribute matching REGEX (unparsed)",
            u"--id-regex=REGEX : extract text from elements with id attribute matching REGEX (unparsed)",
            u"--max-head=DUR : audio head has at most DUR seconds",
            u"--max-tail=DUR : audio tail has at most DUR seconds",
            u"--min-head=DUR : audio head has at least DUR seconds",
            u"--min-tail=DUR : audio tail has at least DUR seconds",
            u"--sort=ALGORITHM : sort the matched element id attributes using ALGORITHM (lexicographic, numeric, unsorted)"
        ]
    }

    def perform_command(self):
        """
        Perform command and return the appropriate exit code.

        :rtype: int
        """
        if len(self.actual_arguments) < 4:
            return self.print_help()
        text_format = gf.safe_unicode(self.actual_arguments[0])
        if text_format == u"list":
            text = gf.safe_unicode(self.actual_arguments[1])
        elif text_format in [u"parsed", u"plain", u"subtitles", u"unparsed"]:
            text = self.actual_arguments[1]
            if not self.check_input_file(text):
                return self.ERROR_EXIT_CODE
        else:
            return self.print_help()

        id_regex = self.has_option_with_value(u"--id-regex")
        class_regex = self.has_option_with_value(u"--class-regex")
        sort = self.has_option_with_value(u"--sort")
        parameters = {
            gc.PPN_JOB_IS_TEXT_UNPARSED_ID_REGEX : id_regex,
            gc.PPN_JOB_IS_TEXT_UNPARSED_CLASS_REGEX : class_regex,
            gc.PPN_JOB_IS_TEXT_UNPARSED_ID_SORT : sort
        }
        if (text_format == u"unparsed") and (id_regex is None) and (class_regex is None):
            self.print_error(u"You must specify --id-regex and/or --class-regex for unparsed format")
            return self.ERROR_EXIT_CODE

        language = gf.safe_unicode(self.actual_arguments[2])
        if (not language in Language.ALLOWED_VALUES) and (not self.rconf["allow_unlisted_languages"]):
            self.print_error(u"Language '%s' is not supported" % (language))
            return self.ERROR_EXIT_CODE

        audio_file_path = self.actual_arguments[3]
        if not self.check_input_file(audio_file_path):
            return self.ERROR_EXIT_CODE

        text_file = self.get_text_file(text_format, text, parameters)
        if text_file is None:
            self.print_error(u"Unable to build a TextFile from the given parameters")
            return self.ERROR_EXIT_CODE
        text_file.set_language(language)
        self.print_info(u"Read input text with %d fragments" % (len(text_file)))

        self.print_info(u"Reading audio...")
        try:
            tmp_handler, tmp_file_path = gf.tmp_file(suffix=u".wav", root=self.rconf["tmp_path"])
            converter = FFMPEGWrapper(rconf=self.rconf, logger=self.logger)
            converter.convert(audio_file_path, tmp_file_path)
        except Exception as exc:
            self.print_error(u"An unexpected Exception occurred while converting the audio file:")
            self.print_error(u"%s" % exc)
            return self.ERROR_EXIT_CODE
        try:
            audio_file = AudioFileMonoWAVE(tmp_file_path, rconf=self.rconf, logger=self.logger)
        except Exception as exc:
            self.print_error(u"An unexpected Exception occurred while reading the converted audio file:")
            self.print_error(u"%s" % exc)
            return self.ERROR_EXIT_CODE
        self.print_info(u"Reading audio... done")

        min_head = gf.safe_float(self.has_option_with_value(u"--min-head"), None)
        max_head = gf.safe_float(self.has_option_with_value(u"--max-head"), None)
        min_tail = gf.safe_float(self.has_option_with_value(u"--min-tail"), None)
        max_tail = gf.safe_float(self.has_option_with_value(u"--max-tail"), None)

        self.print_info(u"Detecting audio interval...")
        start_detector = SD(audio_file, text_file, rconf=self.rconf, logger=self.logger)
        start, end = start_detector.detect_interval(min_head, max_head, min_tail, max_tail)
        self.print_info(u"Detecting audio interval... done")

        self.print_result(audio_file.audio_length, start, end)
        gf.delete_file(tmp_handler, tmp_file_path)
        return self.NO_ERROR_EXIT_CODE

    def print_result(self, audio_len, start, end):
        """
        Print result of SD.

        :param audio_len: the length of the entire audio file, in seconds
        :type  audio_len: float
        :param start: the start position of the spoken text
        :type  start: float
        :param end: the end position of the spoken text
        :type  end: float
        """
        msg = []
        zero = 0
        head_len = start
        text_len = end - start
        tail_len = audio_len - end
        msg.append(u"")
        msg.append(u"Head: %.3f %.3f (%.3f)" % (zero, start, head_len))
        msg.append(u"Text: %.3f %.3f (%.3f)" % (start, end, text_len))
        msg.append(u"Tail: %.3f %.3f (%.3f)" % (end, audio_len, tail_len))
        msg.append(u"")
        zero_h = gf.time_to_hhmmssmmm(0)
        start_h = gf.time_to_hhmmssmmm(start)
        end_h = gf.time_to_hhmmssmmm(end)
        audio_len_h = gf.time_to_hhmmssmmm(audio_len)
        head_len_h = gf.time_to_hhmmssmmm(head_len)
        text_len_h = gf.time_to_hhmmssmmm(text_len)
        tail_len_h = gf.time_to_hhmmssmmm(tail_len)
        msg.append("Head: %s %s (%s)" % (zero_h, start_h, head_len_h))
        msg.append("Text: %s %s (%s)" % (start_h, end_h, text_len_h))
        msg.append("Tail: %s %s (%s)" % (end_h, audio_len_h, tail_len_h))
        msg.append(u"")
        self.print_info(u"\n".join(msg))



def main():
    """
    Execute program.
    """
    RunSDCLI().run(arguments=sys.argv)

if __name__ == '__main__':
    main()






#!/usr/bin/env python
# coding=utf-8

"""
aeneas.cew is a Python C extension to synthesize text with eSpeak
"""

__author__ = "Alberto Pettarin"
__copyright__ = """
    Copyright 2012-2013, Alberto Pettarin (www.albertopettarin.it)
    Copyright 2013-2015, ReadBeyond Srl   (www.readbeyond.it)
    Copyright 2015-2016, Alberto Pettarin (www.albertopettarin.it)
    """
__license__ = "GNU AGPL 3"
__version__ = "1.4.1"
__email__ = "aeneas@readbeyond.it"
__status__ = "Production"






#!/usr/bin/env python
# coding=utf-8

"""
Execute a Task, that is, a pair of audio/text files
and a configuration string.
"""

from __future__ import absolute_import
from __future__ import print_function
import sys

from aeneas.adjustboundaryalgorithm import AdjustBoundaryAlgorithm
from aeneas.downloader import Downloader
from aeneas.executetask import ExecuteTask
from aeneas.idsortingalgorithm import IDSortingAlgorithm
from aeneas.language import Language
from aeneas.syncmap import SyncMapFormat
from aeneas.syncmap import SyncMapHeadTailFormat
from aeneas.task import Task
from aeneas.textfile import TextFileFormat
from aeneas.tools.abstract_cli_program import AbstractCLIProgram
from aeneas.validator import Validator
import aeneas.globalconstants as gc
import aeneas.globalfunctions as gf

__author__ = "Alberto Pettarin"
__copyright__ = """
    Copyright 2012-2013, Alberto Pettarin (www.albertopettarin.it)
    Copyright 2013-2015, ReadBeyond Srl   (www.readbeyond.it)
    Copyright 2015-2016, Alberto Pettarin (www.albertopettarin.it)
    """
__license__ = "GNU AGPL 3"
__version__ = "1.4.1"
__email__ = "aeneas@readbeyond.it"
__status__ = "Production"

class ExecuteTaskCLI(AbstractCLIProgram):
    """
    Execute a Task, that is, a pair of audio/text files
    and a configuration string.
    """

    AUDIO_FILE = gf.relative_path("res/audio.mp3", __file__)

    DEMOS = {
        u"--example-json" : {
            u"description": u"input: plain text, output: JSON",
            u"audio": AUDIO_FILE,
            u"text": gf.relative_path("res/plain.txt", __file__),
            u"config": u"task_language=en|is_text_type=plain|os_task_file_format=json",
            u"syncmap": "output/sonnet.json",
            u"options": u""
        },
        u"--example-srt" : {
            u"description": u"input: subtitles text, output: SRT",
            u"audio": AUDIO_FILE,
            u"text": gf.relative_path("res/subtitles.txt", __file__),
            u"config": u"task_language=en|is_text_type=subtitles|os_task_file_format=srt",
            u"syncmap": "output/sonnet.srt",
            u"options": u""
        },
        u"--example-smil" : {
            u"description": u"input: unparsed text, output: SMIL",
            u"audio": AUDIO_FILE,
            u"text": gf.relative_path("res/page.xhtml", __file__),
            u"config": u"task_language=en|is_text_type=unparsed|is_text_unparsed_id_regex=f[0-9]+|is_text_unparsed_id_sort=numeric|os_task_file_format=smil|os_task_file_smil_audio_ref=p001.mp3|os_task_file_smil_page_ref=p001.xhtml",
            u"syncmap": "output/sonnet.smil",
            u"options": u""
        },
        u"--example-youtube" : {
            u"description": u"input: audio from YouTube, output: TXT",
            u"audio": "https://www.youtube.com/watch?v=rU4a7AA8wM0",
            u"text": gf.relative_path("res/plain.txt", __file__),
            u"config": u"task_language=en|is_text_type=plain|os_task_file_format=txt",
            u"syncmap": "output/sonnet.txt",
            u"options": u"-y"
        }
    }

    PARAMETERS = [
        u"  task_language                           : language (*)",
        u"",
        u"  is_audio_file_detect_head_max           : detect audio head, at most this many seconds",
        u"  is_audio_file_detect_head_min           : detect audio head, at least this many seconds",
        u"  is_audio_file_detect_tail_max           : detect audio tail, at most this many seconds",
        u"  is_audio_file_detect_tail_min           : detect audio tail, at least this many seconds",
        u"  is_audio_file_head_length               : ignore this many seconds from the begin of the audio file",
        u"  is_audio_file_process_length            : process this many seconds of the audio file",
        u"  is_audio_file_tail_length               : ignore this many seconds from the end of the audio file",
        u"",
        u"  is_text_type                            : input text format (*)",
        u"  is_text_unparsed_class_regex            : regex matching class attributes (unparsed)",
        u"  is_text_unparsed_id_regex               : regex matching id attributes (unparsed)",
        u"  is_text_unparsed_id_sort                : sort matched elements by id (unparsed) (*)",
        u"  is_text_file_ignore_regex               : ignore text matched by regex for audio alignment purposes",
        u"  is_text_file_transliterate_map          : apply the given transliteration map for audio alignment purposes",
        u"",
        u"  os_task_file_format                     : output sync map format (*)",
        u"  os_task_file_id_regex                   : id regex for the output sync map (subtitles, plain)",
        u"  os_task_file_head_tail_format           : format audio head/tail (*)",
        u"  os_task_file_smil_audio_ref             : value for the audio ref (smil, smilh, smilm)",
        u"  os_task_file_smil_page_ref              : value for the text ref (smil, smilh, smilm)",
        u"",
        u"  task_adjust_boundary_algorithm          : adjust sync map fragments using algorithm (*)",
        u"  task_adjust_boundary_aftercurrent_value : offset value, in seconds (aftercurrent)",
        u"  task_adjust_boundary_beforenext_value   : offset value, in seconds (beforenext)",
        u"  task_adjust_boundary_offset_value       : offset value, in seconds (offset)",
        u"  task_adjust_boundary_percent_value      : percent value, in [0..100], (percent)",
        u"  task_adjust_boundary_rate_value         : max rate, in characters/s (rate, rateaggressive)",
    ]

    VALUES = {
        "task_language" : Language,
        "is_text_type" : TextFileFormat,
        "is_text_unparsed_id_sort" : IDSortingAlgorithm,
        "os_task_file_format" : SyncMapFormat,
        "os_task_file_head_tail_format" : SyncMapHeadTailFormat,
        "task_adjust_boundary_algorithm" : AdjustBoundaryAlgorithm,
    }

    NAME = gf.file_name_without_extension(__file__)

    HELP = {
        "description": u"Execute a Task.",
        "synopsis": [
            u"--examples",
            u"[--example-json|--example-srt|--example-smil]",
            u"--list-parameters",
            u"--list-values PARAM",
            u"AUDIO TEXT CONFIG_STRING OUTPUT_FILE",
            u"YOUTUBE_URL TEXT CONFIG_STRING OUTPUT_FILE -y",
        ],
        "examples": [
            u"--examples"
        ],
        "options": [
            u"--example-json : run example with JSON output",
            u"--example-smil : run example with SMIL output",
            u"--example-srt : run example with SRT output",
            u"--example-youtube : run example with audio downloaded from YouTube",
            u"--examples : print a list of examples",
            u"--keep-audio : do not delete the audio file downloaded from YouTube (-y only)",
            u"--largest-audio : download largest audio stream (-y only)",
            u"--list-parameters : list all parameters",
            u"--list-values=PARAM : list all allowed values for parameter PARAM",
            u"--output-html : output HTML file for fine tuning",
            u"--skip-validator : do not validate the given config string",
            u"-e, --examples : print examples",
            u"-y, --youtube : download audio from YouTube video",
        ]
    }

    def perform_command(self):
        """
        Perform command and return the appropriate exit code.

        :rtype: int
        """
        if len(self.actual_arguments) < 1:
            return self.print_help()

        if self.has_option([u"-e", u"--examples"]):
            return self.print_examples()

        if self.has_option([u"--list-parameters"]):
            return self.print_parameters()

        parameter = self.has_option_with_value(u"--list-values")
        if parameter is not None:
            return self.print_values(parameter)
        elif self.has_option(u"--list-values"):
            return self.print_values(u"?")

        # TODO simplify me

        # NOTE list() is needed for Python3, where keys() is not a list!
        demo = self.has_option(list(self.DEMOS.keys()))
        download_from_youtube = self.has_option([u"-y", u"--youtube"])
        largest_audio = self.has_option(u"--largest-audio")
        keep_audio = self.has_option(u"--keep-audio")
        output_html = self.has_option(u"--output-html")
        validate = not self.has_option(u"--skip-validator")

        if demo:
            validate = False
            for key in self.DEMOS:
                if self.has_option(key):
                    demo_parameters = self.DEMOS[key]
                    audio_file_path = demo_parameters[u"audio"]
                    text_file_path = demo_parameters[u"text"]
                    config_string = demo_parameters[u"config"]
                    sync_map_file_path = demo_parameters[u"syncmap"]
                    if key == u"--example-youtube":
                        download_from_youtube = True
                    break
        else:
            if len(self.actual_arguments) < 4:
                return self.print_help()
            audio_file_path = self.actual_arguments[0]
            text_file_path = self.actual_arguments[1]
            config_string = self.actual_arguments[2]
            sync_map_file_path = self.actual_arguments[3]

        html_file_path = None
        if output_html:
            keep_audio = True
            html_file_path = sync_map_file_path + u".html"

        if download_from_youtube:
            youtube_url = audio_file_path

        if (not download_from_youtube) and (not self.check_input_file(audio_file_path)):
            return self.ERROR_EXIT_CODE
        if not self.check_input_file(text_file_path):
            return self.ERROR_EXIT_CODE
        if not self.check_output_file(sync_map_file_path):
            return self.ERROR_EXIT_CODE
        if (html_file_path is not None) and (not self.check_output_file(html_file_path)):
            return self.ERROR_EXIT_CODE

        self.check_c_extensions()

        if demo:
            msg = []
            msg.append(u"Running example task with arguments:")
            if download_from_youtube:
                msg.append(u"  YouTube URL:   %s" % youtube_url)
            else:
                msg.append(u"  Audio file:    %s" % audio_file_path)
            msg.append(u"  Text file:     %s" % text_file_path)
            msg.append(u"  Config string: %s" % config_string)
            msg.append(u"  Sync map file: %s" % sync_map_file_path)
            self.print_info(u"\n".join(msg))

        if validate:
            self.print_info(u"Validating config string (specify --skip-validator to bypass)...")
            validator = Validator(logger=self.logger)
            result = validator.check_configuration_string(config_string, is_job=False, external_name=True)
            if not result.passed:
                self.print_error(u"The given config string is not valid:")
                self.print_generic(result.pretty_print())
                return self.ERROR_EXIT_CODE
            self.print_info(u"Validating config string... done")

        if download_from_youtube:
            try:
                self.print_info(u"Downloading audio from '%s' ..." % youtube_url)
                downloader = Downloader(logger=self.logger)
                audio_file_path = downloader.audio_from_youtube(
                    youtube_url,
                    download=True,
                    output_file_path=None,
                    largest_audio=largest_audio
                )
                self.print_info(u"Downloading audio from '%s' ... done" % youtube_url)
            except ImportError:
                self.print_error(u"You need to install Pythom module pafy to download audio from YouTube. Run:")
                self.print_error(u"$ sudo pip install pafy")
                return self.ERROR_EXIT_CODE
            except Exception as exc:
                self.print_error(u"An unexpected Exception occurred while downloading audio from YouTube:")
                self.print_error(u"%s" % exc)
                return self.ERROR_EXIT_CODE

        try:
            self.print_info(u"Creating task...")
            task = Task(config_string, logger=self.logger)
            task.audio_file_path_absolute = audio_file_path
            task.text_file_path_absolute = text_file_path
            task.sync_map_file_path_absolute = sync_map_file_path
            self.print_info(u"Creating task... done")
        except Exception as exc:
            self.print_error(u"An unexpected Exception occurred while creating the task:")
            self.print_error(u"%s" % exc)
            return self.ERROR_EXIT_CODE

        try:
            self.print_info(u"Executing task...")
            executor = ExecuteTask(task=task, rconf=self.rconf, logger=self.logger)
            executor.execute()
            self.print_info(u"Executing task... done")
        except Exception as exc:
            self.print_error(u"An unexpected Exception occurred while executing the task:")
            self.print_error(u"%s" % exc)
            return self.ERROR_EXIT_CODE

        try:
            self.print_info(u"Creating output sync map file...")
            path = task.output_sync_map_file()
            self.print_info(u"Creating output sync map file... done")
            self.print_info(u"Created %s" % path)
        except Exception as exc:
            self.print_error(u"An unexpected Exception occurred while writing the sync map file:")
            self.print_error(u"%s" % exc)
            return self.ERROR_EXIT_CODE

        if output_html:
            try:
                parameters = {}
                parameters[gc.PPN_TASK_OS_FILE_FORMAT] = task.configuration["o_format"]
                parameters[gc.PPN_TASK_OS_FILE_SMIL_AUDIO_REF] = task.configuration["o_smil_audio_ref"]
                parameters[gc.PPN_TASK_OS_FILE_SMIL_PAGE_REF] = task.configuration["o_smil_page_ref"]
                self.print_info(u"Creating output HTML file...")
                task.sync_map.output_html_for_tuning(audio_file_path, html_file_path, parameters)
                self.print_info(u"Creating output HTML file... done")
                self.print_info(u"Created %s" % html_file_path)
            except Exception as exc:
                self.print_error(u"An unexpected Exception occurred while writing the HTML file:")
                self.print_error(u"%s" % exc)
                return self.ERROR_EXIT_CODE

        if download_from_youtube:
            if keep_audio:
                self.print_info(u"Option --keep-audio set: keeping downloaded file '%s'" % audio_file_path)
            else:
                gf.delete_file(None, audio_file_path)

        return self.NO_ERROR_EXIT_CODE



    def print_examples(self):
        """
        Print the examples and exit.
        """
        msg = []
        i = 1
        for key in sorted(self.DEMOS.keys()):
            example = self.DEMOS[key]
            msg.append(u"Example %d (%s)" % (i, example[u"description"]))
            msg.append(u"  $ CONFIG_STRING=\"%s\"" % (example[u"config"]))
            msg.append(u"  $ python -m aeneas.tools.%s %s %s \"$CONFIG_STRING\" %s %s" % (
                self.NAME,
                example[u"audio"],
                example[u"text"],
                example[u"syncmap"],
                example[u"options"]
            ))
            msg.append(u"  or")
            msg.append(u"  $ python -m aeneas.tools.%s %s" % (self.NAME, key))
            msg.append(u"")
            i += 1
        self.print_generic(u"\n" + u"\n".join(msg) + u"\n")
        return self.HELP_EXIT_CODE

    def print_parameters(self):
        """
        Print the list of parameters and exit.
        """
        self.print_info(u"You can use --list-values=PARAM on parameters marked by (*)")
        self.print_info(u"Available parameters:")
        self.print_generic(u"\n" + u"\n".join(self.PARAMETERS) + u"\n")
        return self.HELP_EXIT_CODE

    def print_values(self, parameter):
        """
        Print the list of values for the given parameter and exit.

        If ``parameter`` is invalid, print the list of
        parameter names that have allowed values.

        :param parameter: the parameter name
        :type  parameter: Unicode string
        """
        if parameter in self.VALUES:
            self.print_info(u"Available values for parameter '%s':" % parameter)
            self.print_generic(u", ".join(self.VALUES[parameter].ALLOWED_VALUES))
            return self.HELP_EXIT_CODE
        if parameter not in [u"?", u""]:
            self.print_error(u"Invalid parameter name '%s'" % parameter)
        self.print_info(u"The values for the following parameters can be enumerated:")
        self.print_info(u", ".join(sorted(self.VALUES.keys())))
        return self.HELP_EXIT_CODE



def main():
    """
    Execute program.
    """
    ExecuteTaskCLI().run(arguments=sys.argv)

if __name__ == '__main__':
    main()






#!/usr/bin/env python
# coding=utf-8

"""
Synthesize text using the ``espeak`` wrapper.
"""

from __future__ import absolute_import
from __future__ import print_function
import sys

from aeneas.espeakwrapper import ESPEAKWrapper
from aeneas.language import Language
from aeneas.textfile import TextFile
from aeneas.tools.abstract_cli_program import AbstractCLIProgram
import aeneas.globalfunctions as gf

__author__ = "Alberto Pettarin"
__copyright__ = """
    Copyright 2012-2013, Alberto Pettarin (www.albertopettarin.it)
    Copyright 2013-2015, ReadBeyond Srl   (www.readbeyond.it)
    Copyright 2015-2016, Alberto Pettarin (www.albertopettarin.it)
    """
__license__ = "GNU AGPL 3"
__version__ = "1.4.1"
__email__ = "aeneas@readbeyond.it"
__status__ = "Production"

class ESPEAKWrapperCLI(AbstractCLIProgram):
    """
    Synthesize text using the ``espeak`` wrapper.
    """
    OUTPUT_FILE = "output/sonnet.wav"
    TEXT = u"From fairest creatures we desire increase"
    TEXT_MULTI = u"From|fairest|creatures|we|desire|increase"

    NAME = gf.file_name_without_extension(__file__)

    HELP = {
        "description": u"Synthesize text using the espeak wrapper.",
        "synopsis": [
            u"TEXT LANGUAGE OUTPUT_FILE"
        ],
        "examples": [
            u"\"%s\" en %s" % (TEXT, OUTPUT_FILE),
            u"\"%s\" en %s -m" % (TEXT_MULTI, OUTPUT_FILE)
        ],
        "options": [
            u"-m, --multiple : text contains multiple fragments, separated by a '|' character",
        ]
    }

    def perform_command(self):
        """
        Perform command and return the appropriate exit code.

        :rtype: int
        """
        if len(self.actual_arguments) < 3:
            return self.print_help()
        text = gf.safe_unicode(self.actual_arguments[0])
        language = gf.safe_unicode(self.actual_arguments[1])
        output_file_path = self.actual_arguments[2]
        multiple = self.has_option([u"-m", u"--multiple"])

        if (not language in Language.ALLOWED_VALUES) and (not self.rconf["allow_unlisted_languages"]):
            self.print_error(u"Language code '%s' is not allowed." % language)
            return self.ERROR_EXIT_CODE

        if not self.check_output_file(output_file_path):
            return self.ERROR_EXIT_CODE

        try:
            synt = ESPEAKWrapper(rconf=self.rconf, logger=self.logger)
            if multiple:
                tfl = TextFile()
                tfl.read_from_list(text.split("|"))
                tfl.set_language(language)
                synt.synthesize_multiple(
                    tfl,
                    output_file_path
                )
            else:
                synt.synthesize_single(
                    text,
                    language,
                    output_file_path
                )
            self.print_info(u"Created file '%s'" % output_file_path)
            return self.NO_ERROR_EXIT_CODE
        except OSError:
            self.print_error(u"Unable to create file '%s'" % output_file_path)

        return self.ERROR_EXIT_CODE



def main():
    """
    Execute program.
    """
    ESPEAKWrapperCLI().run(arguments=sys.argv)

if __name__ == '__main__':
    main()






#!/usr/bin/env python
# coding=utf-8

"""
A structure representing a job, that is,
a collection of related Tasks.
"""

from __future__ import absolute_import
from __future__ import print_function

from aeneas.configurationobject import ConfigurationObject
from aeneas.logger import Logger
import aeneas.globalconstants as gc
import aeneas.globalfunctions as gf

__author__ = "Alberto Pettarin"
__copyright__ = """
    Copyright 2012-2013, Alberto Pettarin (www.albertopettarin.it)
    Copyright 2013-2015, ReadBeyond Srl   (www.readbeyond.it)
    Copyright 2015-2016, Alberto Pettarin (www.albertopettarin.it)
    """
__license__ = "GNU AGPL v3"
__version__ = "1.4.1"
__email__ = "aeneas@readbeyond.it"
__status__ = "Production"

class Job(object):
    """
    A structure representing a job, that is,
    a collection of related Tasks.

    :param config_string: the job configuration string
    :type  config_string: Unicode string
    :param logger: the logger object
    :type  logger: :class:`aeneas.logger.Logger`

    :raises TypeError: if ``config_string`` is not ``None`` and
                       not a Unicode string
    """

    TAG = u"Job"

    def __init__(self, config_string=None, logger=None):
        self.logger = logger or Logger()
        self.tasks = []
        self.identifier = gf.uuid_string()
        self.configuration = None
        if config_string is not None:
            self.configuration = JobConfiguration(config_string)

    def _log(self, message, severity=Logger.DEBUG):
        """ Log """
        self.logger.log(message, severity, self.TAG)

    def __len__(self):
        return len(self.tasks)

    def __unicode__(self):
        i = 0
        msg = []
        msg.append(u"%s: '%s'" % (gc.RPN_JOB_IDENTIFIER, self.identifier))
        msg.append(u"Configuration:\n%s" % self.configuration.__unicode__())
        msg.append(u"Tasks:")
        for task in self.tasks:
            msg.append(u"Task %d %s" % (i, task.identifier))
            i += 1
        return u"\n".join(msg)

    def __str__(self):
        return gf.safe_str(self.__unicode__())

    def append_task(self, task):
        """
        Append a task to this job.

        :param task: the task to be appended
        :type  task: :class:`aeneas.task.Task`
        """
        self.tasks.append(task)

    def clear_tasks(self):
        """
        Delete all the tasks of this job.
        """
        self.tasks = []

    @property
    def identifier(self):
        """
        The identifier of the job.

        :rtype: Unicode string
        """
        return self.__identifier
    @identifier.setter
    def identifier(self, value):
        self.__identifier = value



class JobConfiguration(ConfigurationObject):
    """
    A structure representing a configuration for a job, that is,
    a series of directives for I/O and processing the job.

    Allowed keys:

    * ``PPN_JOB_DESCRIPTION``                   or ``description``
    * ``PPN_JOB_LANGUAGE``                      or ``language``
    * ``PPN_JOB_IS_AUDIO_FILE_NAME_REGEX``      or ``i_a_name_regex``
    * ``PPN_JOB_IS_AUDIO_FILE_RELATIVE_PATH``   or ``i_a_relative_path``
    * ``PPN_JOB_IS_HIERARCHY_PREFIX``           or ``i_hierarchy_prefix``
    * ``PPN_JOB_IS_HIERARCHY_TYPE``             or ``i_hierarchy_type``
    * ``PPN_JOB_IS_TASK_DIRECTORY_NAME_REGEX``  or ``i_task_directory_name_regex``
    * ``PPN_JOB_IS_TEXT_FILE_FORMAT``           or ``i_t_format``
    * ``PPN_JOB_IS_TEXT_FILE_NAME_REGEX``       or ``i_t_name_regex``
    * ``PPN_JOB_IS_TEXT_FILE_RELATIVE_PATH``    or ``i_t_relative_path``
    * ``PPN_JOB_IS_TEXT_UNPARSED_CLASS_REGEX``  or ``i_t_unparsed_class_regex``
    * ``PPN_JOB_IS_TEXT_UNPARSED_ID_REGEX``     or ``i_t_unparsed_id_regex``
    * ``PPN_JOB_IS_TEXT_UNPARSED_ID_SORT``      or ``i_t_unparsed_id_sort``
    * ``PPN_JOB_OS_CONTAINER_FORMAT``           or ``o_container_format``
    * ``PPN_JOB_OS_FILE_NAME``                  or ``o_name``
    * ``PPN_JOB_OS_HIERARCHY_PREFIX``           or ``o_hierarchy_prefix``
    * ``PPN_JOB_OS_HIERARCHY_TYPE``             or ``o_hierarchy_type``

    :param config_string: the job configuration string
    :type  config_string: Unicode string

    :raises TypeError: if ``config_string`` is not ``None`` and
                       it is not a Unicode string
    :raises KeyError: if trying to access a key not listed above
    """

    TAG = u"JobConfiguration"

    FIELDS = [
        (gc.PPN_JOB_DESCRIPTION, (None, None, ["description"])),
        (gc.PPN_JOB_LANGUAGE, (None, None, ["language"])),
        (gc.PPN_JOB_IS_AUDIO_FILE_NAME_REGEX, (None, None, ["i_a_name_regex"])),
        (gc.PPN_JOB_IS_AUDIO_FILE_RELATIVE_PATH, (None, None, ["i_a_relative_path"])),
        (gc.PPN_JOB_IS_HIERARCHY_PREFIX, (None, None, ["i_hierarchy_prefix"])),
        (gc.PPN_JOB_IS_HIERARCHY_TYPE, (None, None, ["i_hierarchy_type"])),
        (gc.PPN_JOB_IS_TASK_DIRECTORY_NAME_REGEX, (None, None, ["i_task_directory_name_regex"])),
        (gc.PPN_JOB_IS_TEXT_FILE_FORMAT, (None, None, ["i_t_format"])),
        (gc.PPN_JOB_IS_TEXT_FILE_NAME_REGEX, (None, None, ["i_t_name_regex"])),
        (gc.PPN_JOB_IS_TEXT_FILE_RELATIVE_PATH, (None, None, ["i_t_relative_path"])),
        (gc.PPN_JOB_IS_TEXT_UNPARSED_CLASS_REGEX, (None, None, ["i_t_unparsed_class_regex"])),
        (gc.PPN_JOB_IS_TEXT_UNPARSED_ID_REGEX, (None, None, ["i_t_unparsed_id_regex"])),
        (gc.PPN_JOB_IS_TEXT_UNPARSED_ID_SORT, (None, None, ["i_t_unparsed_id_sort"])),
        (gc.PPN_JOB_OS_CONTAINER_FORMAT, (None, None, ["o_container_format"])),
        (gc.PPN_JOB_OS_FILE_NAME, (None, None, ["o_name"])),
        (gc.PPN_JOB_OS_HIERARCHY_PREFIX, (None, None, ["o_hierarchy_prefix"])),
        (gc.PPN_JOB_OS_HIERARCHY_TYPE, (None, None, ["o_hierarchy_type"])),
    ]

    def __init__(self, config_string=None):
        super(JobConfiguration, self).__init__(config_string)






#!/usr/bin/env python
# coding=utf-8

"""
This module contains the implementation
of dynamic time warping (DTW) algorithms
to align two audio waves, represented by their
Mel-frequency cepstral coefficients (MFCCs).

The two classes provided by this module are:

1. :class:`aeneas.dtw.DTWAlgorithm`
   is an enumeration of the available algorithms.
2. :class:`aeneas.dtw.DTWAligner`
   is the actual feature extractor and aligner.

To align two wave files:

1. build an :class:`aeneas.dtw.DTWAligner` object
   passing the paths of the two wave files
   in the constructor, possibly with custom arguments
   to fine-tune the alignment;
2. call ``compute_mfcc`` to extract the MFCCs of the two wave files;
3. call ``compute_path`` to compute the min cost path between
   the MFCC representations of the two wave files;
4. obtain the map between the two wave files by reading the
   ``computed_map`` property.
"""

from __future__ import absolute_import
from __future__ import division
from __future__ import print_function
import numpy

from aeneas.audiofile import AudioFileMonoWAVE
from aeneas.logger import Logger
from aeneas.runtimeconfiguration import RuntimeConfiguration
import aeneas.globalfunctions as gf

__author__ = "Alberto Pettarin"
__copyright__ = """
    Copyright 2012-2013, Alberto Pettarin (www.albertopettarin.it)
    Copyright 2013-2015, ReadBeyond Srl   (www.readbeyond.it)
    Copyright 2015-2016, Alberto Pettarin (www.albertopettarin.it)
    """
__license__ = "GNU AGPL v3"
__version__ = "1.4.1"
__email__ = "aeneas@readbeyond.it"
__status__ = "Production"

class DTWAlgorithm(object):
    """
    Enumeration of the DTW algorithms that can be used
    for the alignment of two audio waves.
    """

    EXACT = "exact"
    """ Classical (exact) DTW algorithm.

    This implementation has ``O(nm)`` time and space complexity,
    where ``n`` (respectively, ``m``) is the number of MFCCs
    of the real (respectively, synthesized) wave. """

    STRIPE = "stripe"
    """ DTW algorithm restricted to a stripe around the main diagonal
    (Sakoe-Chiba Band), for optimized memory usage and processing.

    Note that this is an heuristic approximation of the optimal (exact) path.

    This implementation has ``O(nd)`` time and space complexity,
    where ``n`` is the number of MFCCs of the real wave,
    and ``d`` is the number of MFCCs
    corresponding to the margin. """

    ALLOWED_VALUES = [EXACT, STRIPE]
    """ List of all the allowed values """



class DTWAligner(object):
    """
    The MFCC extractor and wave aligner.

    :param real_wave_path: the path to the real wav file (must be mono!)
    :type  real_wave_path: string (path)
    :param synt_wave_path: the path to the synthesized wav file (must be mono!)
    :type  synt_wave_path: string (path)
    :param rconf: a runtime configuration. Default: ``None``, meaning that
                  default settings will be used.
    :type  rconf: :class:`aeneas.runtimeconfiguration.RuntimeConfiguration`
    :param logger: the logger object
    :type  logger: :class:`aeneas.logger.Logger`

    :raise ValueError: if ``real_wave_path`` or ``synt_wave_path`` is ``None``
                       or it does not exist, or if ``algorithm`` is not an allowed value
    """

    TAG = u"DTWAligner"

    def __init__(self, real_wave_path=None, synt_wave_path=None, rconf=None, logger=None):
        if (real_wave_path is not None) and (not gf.file_can_be_read(real_wave_path)):
            raise ValueError("Real wave cannot be read")
        if (synt_wave_path is not None) and (not gf.file_can_be_read(synt_wave_path)):
            raise ValueError("Synt wave cannot be read")
        if (rconf is not None) and (rconf["dtw_algorithm"] not in DTWAlgorithm.ALLOWED_VALUES):
            raise ValueError("Algorithm value not allowed")
        self.logger = logger or Logger()
        self.rconf = rconf or RuntimeConfiguration()
        self.real_wave_path = real_wave_path
        self.synt_wave_path = synt_wave_path
        self.real_wave_full_mfcc = None
        self.synt_wave_full_mfcc = None
        self.real_wave_length = None
        self.synt_wave_length = None
        self.computed_path = None

    def _log(self, message, severity=Logger.DEBUG):
        """ Log """
        self.logger.log(message, severity, self.TAG)

    @property
    def real_wave_full_mfcc(self):
        """
        MFCCs of the real wave, including the 0-th.

        :rtype: numpy 2D array

        .. versionadded:: 1.1.0
        """
        return self.__real_wave_full_mfcc

    @real_wave_full_mfcc.setter
    def real_wave_full_mfcc(self, real_wave_full_mfcc):
        self.__real_wave_full_mfcc = real_wave_full_mfcc

    @property
    def real_wave_length(self):
        """
        The length, in seconds, of the real wave.

        :rtype: float

        .. versionadded:: 1.1.0
        """
        return self.__real_wave_length

    @real_wave_length.setter
    def real_wave_length(self, real_wave_length):
        self.__real_wave_length = real_wave_length

    @property
    def synt_wave_full_mfcc(self):
        """
        MFCCs of the synthesized wave, including the 0-th.

        :rtype: numpy 2D array

        .. versionadded:: 1.1.0
        """
        return self.__synt_wave_full_mfcc

    @synt_wave_full_mfcc.setter
    def synt_wave_full_mfcc(self, synt_wave_full_mfcc):
        self.__synt_wave_full_mfcc = synt_wave_full_mfcc

    @property
    def synt_wave_length(self):
        """
        The length, in seconds, of the synthesized wave.

        :rtype: float

        .. versionadded:: 1.1.0
        """
        return self.__synt_wave_length

    @synt_wave_length.setter
    def synt_wave_length(self, synt_wave_length):
        self.__synt_wave_length = synt_wave_length

    def compute_mfcc(self, real_wave=True, synt_wave=True):
        """
        Compute the MFCCs of the two waves,
        and store them internally.

        :param real_wave: if ``True``, extract MFCCs for the real wave
        :type  real_wave: bool
        :param synt_wave: if ``True``, extract MFCCs for the synt wave
        :type  synt_wave: bool

        :raise OSError: if the real or synt wave file cannot be read
        """
        if real_wave:
            if not gf.file_can_be_read(self.real_wave_path):
                raise OSError("Real wave path is None or it cannot be read")
            self._log(u"Computing MFCCs for real wave...")
            wave = AudioFileMonoWAVE(self.real_wave_path, rconf=self.rconf, logger=self.logger)
            wave.extract_mfcc()
            self.real_wave_full_mfcc = wave.audio_mfcc
            self.real_wave_length = wave.audio_length
            self._log(u"Computing MFCCs for real wave... done")

        if synt_wave:
            if not gf.file_can_be_read(self.synt_wave_path):
                raise OSError("Synt wave path is None or it cannot be read")
            self._log(u"Computing MFCCs for synt wave...")
            wave = AudioFileMonoWAVE(self.synt_wave_path, rconf=self.rconf, logger=self.logger)
            wave.extract_mfcc()
            self.synt_wave_full_mfcc = wave.audio_mfcc
            self.synt_wave_length = wave.audio_length
            self._log(u"Computing MFCCs for synt wave... done")

    def compute_accumulated_cost_matrix(self):
        """
        Compute the accumulated cost matrix,
        and return it.

        :rtype: numpy 2D array

        :raise RuntimeError: if both the C extension and
                             the pure Python code did not succeed.

        .. versionadded:: 1.2.0
        """
        dtw = self._setup_dtw()
        self._log(u"Returning accumulated cost matrix")
        return dtw.compute_accumulated_cost_matrix()

    def compute_path(self):
        """
        Compute the min cost path between the two waves,
        and store it internally.

        :raise RuntimeError: if both the C extension and
                             the pure Python code did not succeed.
        """
        dtw = self._setup_dtw()
        self._log(u"Computing path...")
        self.computed_path = dtw.compute_path()
        self._log(u"Computing path... done")

    def _setup_dtw(self):
        """ Setup DTW object """
        # setup
        algorithm = self.rconf["dtw_algorithm"]
        delta = int(2 * self.rconf["dtw_margin"] / self.rconf["mfcc_win_shift"])
        mfcc2_length = self.synt_wave_full_mfcc.shape[1]
        self._log([u"Requested algorithm: '%s'", algorithm])
        self._log([u"delta = %d", delta])
        self._log([u"m = %d", mfcc2_length])
        # check if delta is >= length of synt wave
        if mfcc2_length <= delta:
            self._log(u"We have mfcc2_length <= delta")
            if (self.rconf["c_ext"]) and (gf.can_run_c_extension()):
                # the C code can be run: since it is still faster, do not run EXACT
                self._log(u"C extensions enabled and loaded: not selecting EXACT algorithm")
            else:
                self._log(u"Selecting EXACT algorithm")
                algorithm = DTWAlgorithm.EXACT

        # execute the selected algorithm
        if algorithm == DTWAlgorithm.EXACT:
            self._log(u"Computing with EXACT algo")
            dtw = DTWExact(
                self.real_wave_full_mfcc,
                self.synt_wave_full_mfcc,
                self.logger
            )
        else:
            self._log(u"Computing with STRIPE algo")
            dtw = DTWStripe(
                self.real_wave_full_mfcc,
                self.synt_wave_full_mfcc,
                delta,
                self.logger
            )
        return dtw

    @property
    def computed_map(self):
        """
        Return the computed map between the two waves,
        as a list of lists, each being a pair of floats: ::

        [[r_1, s_1], [r_2, s_2], ..., [r_k, s_k]]

        where ``r_i`` are the time instants in the real wave
        and ``s_i`` are the time instants in the synthesized wave,
        and ``k = n + m`` (or ``k = n + d``)
        is the length of the min cost path.

        :rtype: list of pairs of floats (see above)
        """
        result = []
        for i in range(len(self.computed_path)):
            real_time = self.computed_path[i][0] * self.rconf["mfcc_win_shift"]
            synt_time = self.computed_path[i][1] * self.rconf["mfcc_win_shift"]
            result.append([real_time, synt_time])
        return result



class DTWStripe(object):

    TAG = u"DTWStripe"

    def __init__(self, m1, m2, delta, logger=None):
        self.m1 = m1
        self.m2 = m2
        self.delta = delta
        self.logger = logger or Logger()

    def _log(self, message, severity=Logger.DEBUG):
        """ Log """
        self.logger.log(message, severity, self.TAG)

    def compute_accumulated_cost_matrix(self):
        return gf.run_c_extension_with_fallback(
            self._log,
            "cdtw",
            self._compute_acm_c_extension,
            self._compute_acm_pure_python,
            (),
            c_extension=True
        )

    def _compute_acm_c_extension(self):
        self._log(u"Computing acm using C extension...")
        try:
            self._log(u"Importing cdtw...")
            import aeneas.cdtw.cdtw
            self._log(u"Importing cdtw... done")
            # discard first MFCC component
            mfcc1 = self.m1[1:, :]
            mfcc2 = self.m2[1:, :]
            n = mfcc1.shape[1]
            m = mfcc2.shape[1]
            delta = self.delta
            self._log([u"n m delta: %d %d %d", n, m, delta])
            if delta > m:
                self._log(u"Limiting delta to m")
                delta = m
            cost_matrix, centers = aeneas.cdtw.cdtw.compute_cost_matrix_step(mfcc1, mfcc2, delta)
            accumulated_cost_matrix = aeneas.cdtw.cdtw.compute_accumulated_cost_matrix_step(cost_matrix, centers)
            self._log(u"Computing acm using C extension... done")
            return (True, accumulated_cost_matrix)
        except Exception as exc:
            self._log(u"Computing acm using C extension... failed")
            self._log(u"An unexpected exception occurred while running cdtw:", Logger.WARNING)
            self._log([u"%s", exc], Logger.WARNING)
        return (False, None)

    def _compute_acm_pure_python(self):
        self._log(u"Computing acm using pure Python code...")
        try:
            cost_matrix, centers = self._compute_cost_matrix()
            accumulated_cost_matrix = self._compute_accumulated_cost_matrix(cost_matrix, centers)
            self._log(u"Computing acm using pure Python code... done")
            return (True, accumulated_cost_matrix)
        except Exception as exc:
            self._log(u"Computing acm using pure Python code... failed")
            self._log(u"An unexpected exception occurred while running pure Python code:", Logger.WARNING)
            self._log([u"%s", exc], Logger.WARNING)
        return (False, None)

    def compute_path(self):
        return gf.run_c_extension_with_fallback(
            self._log,
            "cdtw",
            self._compute_path_c_extension,
            self._compute_path_pure_python,
            (),
            c_extension=True
        )

    def _compute_path_c_extension(self):
        self._log(u"Computing path using C extension...")
        try:
            self._log(u"Importing cdtw...")
            import aeneas.cdtw.cdtw
            self._log(u"Importing cdtw... done")
            # discard first MFCC component
            mfcc1 = self.m1[1:, :]
            mfcc2 = self.m2[1:, :]
            n = mfcc1.shape[1]
            m = mfcc2.shape[1]
            delta = self.delta
            self._log([u"n m delta: %d %d %d", n, m, delta])
            if delta > m:
                self._log(u"Limiting delta to m")
                delta = m
            best_path = aeneas.cdtw.cdtw.compute_best_path(
                mfcc1,
                mfcc2,
                delta
            )
            self._log(u"Computing path using C extension... done")
            return (True, best_path)
        except Exception as exc:
            self._log(u"Computing path using C extension... failed")
            self._log(u"An unexpected exception occurred while running cdtw:", Logger.WARNING)
            self._log([u"%s", exc], Logger.WARNING)
        return (False, None)

    def _compute_path_pure_python(self):
        self._log(u"Computing path using pure Python code...")
        try:
            cost_matrix, centers = self._compute_cost_matrix()
            accumulated_cost_matrix = self._compute_accumulated_cost_matrix(cost_matrix, centers)
            best_path = self._compute_best_path(accumulated_cost_matrix, centers)
            self._log(u"Computing path using pure Python code... done")
            return (True, best_path)
        except Exception as exc:
            self._log(u"Computing path using pure Python code... failed")
            self._log(u"An unexpected exception occurred while running cdtw:", Logger.WARNING)
            self._log([u"%s", exc], Logger.WARNING)
        return (False, None)

    def _compute_cost_matrix(self):
        self._log(u"Computing cost matrix...")
        # discard first MFCC component
        mfcc1 = self.m1[1:, :]
        mfcc2 = self.m2[1:, :]
        norm2_1 = numpy.sqrt(numpy.sum(mfcc1 ** 2, 0))
        norm2_2 = numpy.sqrt(numpy.sum(mfcc2 ** 2, 0))
        n = mfcc1.shape[1]
        m = mfcc2.shape[1]
        delta = self.delta
        self._log([u"n m delta: %d %d %d", n, m, delta])
        if delta > m:
            self._log(u"Limiting delta to m")
            delta = m
        cost_matrix = numpy.zeros((n, delta))
        centers = numpy.zeros(n)
        for i in range(n):
            # center j at row i
            center_j = (m * i) // n
            #self._log([u"Center at row %d is %d", i, center_j])
            range_start = max(0, center_j - (delta // 2))
            range_end = range_start + delta
            if range_end > m:
                range_end = m
                range_start = range_end - delta
            centers[i] = range_start
            #self._log([u"Range at row %d is %d %d", i, range_start, range_end])
            for j in range(range_start, range_end):
                tmp = mfcc1[:, i].transpose().dot(mfcc2[:, j])
                tmp /= norm2_1[i] * norm2_2[j]
                cost_matrix[i][j - range_start] = 1 - tmp
        self._log(u"Computing cost matrix... done")
        return (cost_matrix, centers)

    def _compute_accumulated_cost_matrix(self, cost_matrix, centers):
        # create accumulated cost matrix
        #
        # a[i][j] = c[i][j] + min(c[i-1][j-1], c[i-1][j], c[i][j-1])
        #
        return self._compute_acm_in_place(cost_matrix, centers)

    def _compute_acm_in_place(self, cost_matrix, centers):
        self._log(u"Computing the acm with the in-place algorithm...")
        n, delta = cost_matrix.shape
        self._log([u"n delta: %d %d", n, delta])
        current_row = numpy.copy(cost_matrix[0, :])
        #cost_matrix[0][0] = current_row[0]
        for j in range(1, delta):
            cost_matrix[0][j] = current_row[j] + cost_matrix[0][j-1]
        # fill table
        for i in range(1, n):
            current_row = numpy.copy(cost_matrix[i, :])
            offset = centers[i] - centers[i-1]
            for j in range(delta):
                cost0 = numpy.inf
                if (j+offset) < delta:
                    cost0 = cost_matrix[i-1][j+offset]
                cost1 = numpy.inf
                if j > 0:
                    cost1 = cost_matrix[i][j-1]
                cost2 = numpy.inf
                if ((j+offset-1) < delta) and ((j+offset-1) >= 0):
                    cost2 = cost_matrix[i-1][j+offset-1]
                cost_matrix[i][j] = current_row[j] + min(cost0, cost1, cost2)
        self._log(u"Computing the acm with the in-place algorithm... done")
        return cost_matrix

    # DISABLED
    #def _compute_acm_not_in_place(self, cost_matrix, centers):
    #    self._log(u"Computing the acm with the not-in-place algorithm...")
    #    acc_matrix = numpy.zeros(cost_matrix.shape)
    #    n, delta = acc_matrix.shape
    #    self._log([u"n delta: %d %d", n, delta])
    #    # first row
    #    acc_matrix[0][0] = cost_matrix[0][0]
    #    for j in range(1, delta):
    #        acc_matrix[0][j] = acc_matrix[0][j-1] + cost_matrix[0][j]
    #    # fill table
    #    for i in range(1, n):
    #        offset = centers[i] - centers[i-1]
    #        for j in range(delta):
    #            cost0 = numpy.inf
    #            if (j+offset) < delta:
    #                cost0 = acc_matrix[i-1][j+offset]
    #            cost1 = numpy.inf
    #            if j > 0:
    #                cost1 = acc_matrix[i][j-1]
    #            cost2 = numpy.inf
    #            if ((j+offset-1) < delta) and ((j+offset-1) >= 0):
    #                cost2 = acc_matrix[i-1][j+offset-1]
    #            acc_matrix[i][j] = cost_matrix[i][j] + min(cost0, cost1, cost2)
    #    self._log(u"Computing the acm with the not-in-place algorithm... done")
    #    return acc_matrix

    def _compute_best_path(self, acc_matrix, centers):
        self._log(u"Computing best path...")
        # get dimensions
        n, delta = acc_matrix.shape
        self._log([u"n delta: %d %d", n, delta])
        i = n - 1
        j = delta - 1 + centers[i]
        path = [(i, j)]
        # compute best (min cost) path
        while (i > 0) or (j > 0):
            if i == 0:
                path.append((0, j-1))
                j -= 1
            elif j == 0:
                path.append((i-1, 0))
                i -= 1
            else:
                offset = centers[i] - centers[i-1]
                r_j = j - centers[i]
                cost0 = numpy.inf
                if (r_j+offset) < delta:
                    cost0 = acc_matrix[i-1][r_j+offset]
                cost1 = numpy.inf
                if r_j > 0:
                    cost1 = acc_matrix[i][r_j-1]
                cost2 = numpy.inf
                if (r_j > 0) and ((r_j+offset-1) < delta) and ((r_j+offset-1) >= 0):
                    cost2 = acc_matrix[i-1][r_j+offset-1]
                costs = [
                    cost0,
                    cost1,
                    cost2
                ]
                moves = [
                    (i-1, j),
                    (i, j-1),
                    (i-1, j-1)
                ]
                min_cost = numpy.argmin(costs)
                #self._log([u"Selected min cost move %d", min_cost])
                min_move = moves[min_cost]
                path.append(min_move)
                i, j = min_move
        # reverse path and return
        path.reverse()
        self._log(u"Computing best path... done")
        return path



class DTWExact(object):

    TAG = u"DTWExact"

    def __init__(self, m1, m2, logger=None):
        self.m1 = m1
        self.m2 = m2
        self.logger = logger or Logger()

    def _log(self, message, severity=Logger.DEBUG):
        """ Log """
        self.logger.log(message, severity, self.TAG)

    def compute_accumulated_cost_matrix(self):
        self._log(u"Computing acm using pure Python code...")
        cost_matrix = self._compute_cost_matrix()
        accumulated_cost_matrix = self._compute_accumulated_cost_matrix(cost_matrix)
        self._log(u"Computing acm using pure Python code... done")
        return accumulated_cost_matrix

    def compute_path(self):
        self._log(u"Computing path using pure Python code...")
        accumulated_cost_matrix = self.compute_accumulated_cost_matrix()
        best_path = self._compute_best_path(accumulated_cost_matrix)
        self._log(u"Computing path using pure Python code... done")
        return best_path

    def _compute_cost_matrix(self):
        self._log(u"Computing cost matrix...")
        # discard first MFCC component
        mfcc1 = self.m1[1:, :]
        mfcc2 = self.m2[1:, :]
        norm2_1 = numpy.sqrt(numpy.sum(mfcc1 ** 2, 0))
        norm2_2 = numpy.sqrt(numpy.sum(mfcc2 ** 2, 0))
        # compute dot product
        self._log(u"Computing matrix with transpose+dot...")
        cost_matrix = mfcc1.transpose().dot(mfcc2)
        self._log(u"Computing matrix with transpose+dot... done")
        # normalize
        self._log(u"Normalizing matrix...")
        norm_matrix = numpy.outer(norm2_1, norm2_2)
        cost_matrix = 1 - (cost_matrix / norm_matrix)
        self._log(u"Normalizing matrix... done")
        self._log(u"Computing cost matrix... done")
        return cost_matrix

    def _compute_accumulated_cost_matrix(self, cost_matrix):
        # create accumulated cost matrix
        #
        # a[i][j] = c[i][j] + min(c[i-1][j-1], c[i-1][j], c[i][j-1])
        #
        return self._compute_acm_in_place(cost_matrix)

    def _compute_acm_in_place(self, cost_matrix):
        self._log(u"Computing the acm with the in-place algorithm...")
        n, m = cost_matrix.shape
        self._log([u"n m: %d %d", n, m])
        current_row = numpy.copy(cost_matrix[0, :])
        #cost_matrix[0][0] = current_row[0]
        for j in range(1, m):
            cost_matrix[0][j] = current_row[j] + cost_matrix[0][j-1]
        for i in range(1, n):
            current_row = numpy.copy(cost_matrix[i, :])
            cost_matrix[i][0] = cost_matrix[i-1][0] + current_row[0]
            for j in range(1, m):
                cost_matrix[i][j] = current_row[j] + min(
                    cost_matrix[i-1][j],
                    cost_matrix[i][j-1],
                    cost_matrix[i-1][j-1]
                )
        self._log(u"Computing the acm with the in-place algorithm... done")
        return cost_matrix

    # DISABLED
    #def _compute_acm_not_in_place(self, cost_matrix):
    #    self._log(u"Computing the acm with the not-in-place algorithm...")
    #    acc_matrix = numpy.zeros(cost_matrix.shape)
    #    n, m = acc_matrix.shape
    #    self._log([u"n m: %d %d", n, m])
    #    acc_matrix[0][0] = cost_matrix[0][0]
    #    for j in range(1, m):
    #        acc_matrix[0][j] = acc_matrix[0][j-1] + cost_matrix[0][j]
    #    for i in range(1, n):
    #        acc_matrix[i][0] = acc_matrix[i-1][0] + cost_matrix[i][0]
    #    for i in range(1, n):
    #        for j in range(1, m):
    #            acc_matrix[i][j] = cost_matrix[i][j] + min(
    #                acc_matrix[i-1][j],
    #                acc_matrix[i][j-1],
    #                acc_matrix[i-1][j-1]
    #            )
    #    self._log(u"Computing the acm with the not-in-place algorithm... done")
    #    return acc_matrix

    def _compute_best_path(self, acc_matrix):
        self._log(u"Computing best path...")
        # get dimensions
        n, m = acc_matrix.shape
        self._log([u"n m: %d %d", n, m])
        i = n - 1
        j = m - 1
        path = [(i, j)]
        # compute best (min cost) path
        while (i > 0) or (j > 0):
            if i == 0:
                path.append((0, j-1))
                j -= 1
            elif j == 0:
                path.append((i-1, 0))
                i -= 1
            else:
                costs = [
                    acc_matrix[i-1][j],
                    acc_matrix[i][j-1],
                    acc_matrix[i-1][j-1]
                ]
                moves = [
                    (i-1, j),
                    (i, j-1),
                    (i-1, j-1)
                ]
                min_cost = numpy.argmin(costs)
                #self._log([u"Selected min cost move %d", min_cost])
                min_move = moves[min_cost]
                path.append(min_move)
                i, j = min_move
        # reverse path and return
        path.reverse()
        self._log(u"Computing best path... done")
        return path






#!/usr/bin/env python
# coding=utf-8

"""
Wrapper around ``espeak`` to synthesize text into a ``wav`` audio file.
"""

from __future__ import absolute_import
from __future__ import print_function
import subprocess

from aeneas.audiofile import AudioFileMonoWAVE
from aeneas.audiofile import AudioFileUnsupportedFormatError
from aeneas.language import Language
from aeneas.logger import Logger
from aeneas.runtimeconfiguration import RuntimeConfiguration
import aeneas.globalfunctions as gf

__author__ = "Alberto Pettarin"
__copyright__ = """
    Copyright 2012-2013, Alberto Pettarin (www.albertopettarin.it)
    Copyright 2013-2015, ReadBeyond Srl   (www.readbeyond.it)
    Copyright 2015-2016, Alberto Pettarin (www.albertopettarin.it)
    """
__license__ = "GNU AGPL v3"
__version__ = "1.4.1"
__email__ = "aeneas@readbeyond.it"
__status__ = "Production"

class ESPEAKWrapper(object):
    """
    Wrapper around ``espeak`` to synthesize text into a ``wav`` audio file.

    It will perform one or more calls like ::

        $ espeak -v language_code -w /tmp/output_file.wav < text

    In case of multiple text fragments, the resulting wav files
    will be joined together.

    :param logger: the logger object
    :type  logger: :class:`aeneas.logger.Logger`
    """

    TAG = u"ESPEAKWrapper"

    def __init__(self, rconf=None, logger=None):
        self.logger = logger or Logger()
        self.rconf = rconf or RuntimeConfiguration()

    def _log(self, message, severity=Logger.DEBUG):
        """ Log """
        self.logger.log(message, severity, self.TAG)

    def _replace_language(self, language):
        """
        Mock support for a given language by
        synthesizing using a similar language.

        :param language: the requested language
        :type  language: :class:`aeneas.language.Language` enum
        :rtype: :class:`aeneas.language.Language` enum
        """
        if language == Language.UK:
            self._log([u"Replaced '%s' with '%s'", Language.UK, Language.RU])
            return Language.RU
        return language

    def synthesize_multiple(
            self,
            text_file,
            output_file_path,
            quit_after=None,
            backwards=False
    ):
        """
        Synthesize the text contained in the given fragment list
        into a ``wav`` file.

        :param text_file: the text file to be synthesized
        :type  text_file: :class:`aeneas.textfile.TextFile`
        :param output_file_path: the path to the output audio file
        :type  output_file_path: string (path)
        :param quit_after: stop synthesizing as soon as
                           reaching this many seconds
        :type  quit_after: float
        :param backwards: synthesizing from the end of the text file
        :type  backwards: bool
        :rtype: tuple (anchors, total_time, num_chars)

        :raise TypeError: if ``text_file`` is ``None`` or
                          one of the text fragments is not a ``unicode`` object
        :raise ValueError: if ``rconf["allow_unlisted_languages"]`` is ``False`` and
                           a fragment has its language code not listed in
                           :class:`aeneas.language.Language`
        :raise OSError: if output file cannot be written to ``output_file_path``
        :raise RuntimeError: if both the C extension and
                             the pure Python code did not succeed.
        """
        # check that text_file is not None
        if text_file is None:
            self._log(u"text_file is None", Logger.CRITICAL)
            raise TypeError("text_file is None")

        # check that the lines in the text file all have
        # a supported language code and unicode type
        if not self.rconf["allow_unlisted_languages"]:
            for fragment in text_file.fragments:
                if fragment.language not in Language.ALLOWED_VALUES:
                    self._log([u"Language '%s' is not allowed", fragment.language], Logger.CRITICAL)
                    raise ValueError("Language code not allowed")
        for fragment in text_file.fragments:
            for line in fragment.lines:
                if not gf.is_unicode(line):
                    self._log(u"Text file must contain only unicode strings", Logger.CRITICAL)
                    raise TypeError("Text file must contain only unicode strings")

        # log parameters
        if quit_after is not None:
            self._log([u"Quit after reaching %.3f", quit_after])
        if backwards:
            self._log(u"Synthesizing backwards")

        # check that output_file_path can be written
        if not gf.file_can_be_written(output_file_path):
            self._log([u"Cannot write output file to '%s'", output_file_path], Logger.CRITICAL)
            raise OSError("Cannot write output file")

        return gf.run_c_extension_with_fallback(
            self._log,
            "cew",
            self._synthesize_multiple_c_extension,
            self._synthesize_multiple_pure_python,
            (text_file, output_file_path, quit_after, backwards),
            c_extension=self.rconf["c_ext"]
        )

    def _synthesize_multiple_c_extension(
            self,
            text_file,
            output_file_path,
            quit_after=None,
            backwards=False
    ):
        self._log(u"Synthesizing using C extension...")

        # convert parameters from Python values to C values
        try:
            c_quit_after = float(quit_after)
        except TypeError:
            c_quit_after = 0.0
        c_backwards = 0
        if backwards:
            c_backwards = 1
        self._log([u"output_file_path: %s", output_file_path])
        self._log([u"c_quit_after:     %.3f", c_quit_after])
        self._log([u"c_backwards:      %d", c_backwards])
        self._log(u"Preparing c_text...")
        c_text = []
        fragments = text_file.fragments
        for fragment in fragments:
            f_lang = fragment.language
            f_text = fragment.filtered_text
            if f_lang is None:
                f_lang = Language.EN
            f_lang = self._replace_language(f_lang)
            if f_text is None:
                f_text = u""
            if gf.PY2:
                # Python 2 => pass byte strings
                c_text.append((gf.safe_bytes(f_lang), gf.safe_bytes(f_text)))
            else:
                # Python 3 => pass Unicode strings
                c_text.append((gf.safe_unicode(f_lang), gf.safe_unicode(f_text)))
        self._log(u"Preparing c_text... done")

        # call C extension
        try:
            self._log(u"Importing aeneas.cew...")
            import aeneas.cew.cew
            self._log(u"Importing aeneas.cew... done")
            self._log(u"Calling aeneas.cew...")
            sr, sf, intervals = aeneas.cew.cew.synthesize_multiple(
                output_file_path,
                c_quit_after,
                c_backwards,
                c_text
            )
            self._log(u"Calling aeneas.cew... done")
        except Exception as exc:
            self._log(u"Calling aeneas.cew... failed")
            self._log(u"An unexpected exception occurred while running cew:", Logger.WARNING)
            self._log([u"%s", exc], Logger.WARNING)
            return (False, None)
        self._log([u"sr: %d", sr])
        self._log([u"sf: %d", sf])

        # create output
        anchors = []
        current_time = 0.0
        num_chars = 0
        if backwards:
            fragments = fragments[::-1]
        for i in range(sf):
            # get the correct fragment
            fragment = fragments[i]
            # store for later output
            anchors.append([
                intervals[i][0],
                fragment.identifier,
                fragment.filtered_text
            ])
            # increase the character counter
            num_chars += fragment.characters
            # update current_time
            current_time = intervals[i][1]

        # return output
        # NOTE anchors do not make sense if backwards == True
        self._log([u"Returning %d time anchors", len(anchors)])
        self._log([u"Current time %.3f", current_time])
        self._log([u"Synthesized %d characters", num_chars])
        self._log(u"Synthesizing using C extension... done")
        return (True, (anchors, current_time, num_chars))

    def _synthesize_multiple_pure_python(
            self,
            text_file,
            output_file_path,
            quit_after=None,
            backwards=False
    ):
        def synthesize_and_clean(text, language):
            """
            Synthesize a single fragment, pure Python,
            and immediately remove the temporary file.
            """
            self._log(u"Synthesizing text...")
            handler, tmp_destination = gf.tmp_file(suffix=u".wav", root=self.rconf["tmp_path"])
            result, data = self._synthesize_single_pure_python(
                text=(text + u" "),
                language=language,
                output_file_path=tmp_destination
            )
            self._log([u"Removing temporary file '%s'", tmp_destination])
            gf.delete_file(handler, tmp_destination)
            self._log(u"Synthesizing text... done")
            return data

        self._log(u"Synthesizing using pure Python...")

        try:
            # get sample rate and encoding
            du_nu, sample_rate, encoding, da_nu = synthesize_and_clean(
                u"Dummy text to get sample_rate",
                Language.EN
            )

            # open output file
            output_file = AudioFileMonoWAVE(
                file_path=output_file_path,
                logger=self.logger
            )
            output_file.audio_format = encoding
            output_file.audio_sample_rate = sample_rate

            # create output
            anchors = []
            current_time = 0.0
            num = 0
            num_chars = 0
            fragments = text_file.fragments
            if backwards:
                fragments = fragments[::-1]
            for fragment in fragments:
                # replace language
                language = self._replace_language(fragment.language)
                # synthesize and get the duration of the output file
                self._log([u"Synthesizing fragment %d", num])
                duration, sr_nu, enc_nu, data = synthesize_and_clean(
                    text=fragment.filtered_text,
                    language=language
                )
                # store for later output
                anchors.append([current_time, fragment.identifier, fragment.text])
                # increase the character counter
                num_chars += fragment.characters
                # append/prepend data
                self._log([u"Fragment %d starts at: %f", num, current_time])
                if duration > 0:
                    self._log([u"Fragment %d duration: %f", num, duration])
                    current_time += duration
                    #
                    # NOTE since numpy.append cannot be in place,
                    # it seems that the only alternative to make
                    # this more efficient consists in pre-allocating
                    # the destination array,
                    # possibly truncating or extending it as needed
                    #
                    if backwards:
                        output_file.prepend_data(data)
                    else:
                        output_file.append_data(data)
                else:
                    self._log([u"Fragment %d has zero duration", num])

                # increment fragment counter
                num += 1

                # check if we must stop synthesizing because we have enough audio
                if (quit_after is not None) and (current_time > quit_after):
                    self._log([u"Quitting after reached duration %.3f", current_time])
                    break

            # write output file
            self._log([u"Writing audio file '%s'", output_file_path])
            output_file.write(file_path=output_file_path)
            self._log(u"Synthesizing using pure Python... done")
        except Exception as exc:
            self._log(u"Synthesizing using pure Python... failed")
            self._log(u"An unexpected exception occurred while running pure Python code:", Logger.WARNING)
            self._log([u"%s", exc], Logger.WARNING)
            return (False, None)

        # return output
        # NOTE anchors do not make sense if backwards == True
        self._log([u"Returning %d time anchors", len(anchors)])
        self._log([u"Current time %.3f", current_time])
        self._log([u"Synthesized %d characters", num_chars])
        self._log(u"Synthesizing using pure Python... done")
        return (True, (anchors, current_time, num_chars))

    def synthesize_single(
            self,
            text,
            language,
            output_file_path
    ):
        """
        Create a ``wav`` audio file containing the synthesized text.

        The ``text`` must be a unicode string encodable with UTF-8,
        otherwise ``espeak`` might fail.

        Return the duration of the synthesized audio file, in seconds.

        :param text: the text to synthesize
        :type  text: unicode
        :param language: the language to use
        :type  language: :class:`aeneas.language.Language` enum
        :param output_file_path: the path of the output audio file
        :type  output_file_path: string
        :rtype: float

        :raise TypeError: if ``text`` is ``None`` or it is not a ``unicode`` object
        :raise ValueError: if ``rconf["allow_unlisted_languages"]`` is ``False`` and
                           ``language`` is not listed in
                           :class:`aeneas.language.Language`
        :raise OSError: if output file cannot be written to ``output_file_path``
        :raise RuntimeError: if both the C extension and
                             the pure Python code did not succeed.
        """
        # check that text_file is not None
        if text is None:
            self._log(u"text is None", Logger.CRITICAL)
            raise TypeError("text is None")

        # check that text has unicode type
        if not gf.is_unicode(text):
            self._log(u"text must be a unicode string", Logger.CRITICAL)
            raise TypeError("text must be a unicode string")

        # check that output_file_path can be written
        if not gf.file_can_be_written(output_file_path):
            self._log([u"Cannot write output file to '%s'", output_file_path], Logger.CRITICAL)
            raise OSError("Cannot write output file")

        # check that the requested language is listed in language.py
        if (language not in Language.ALLOWED_VALUES) and (not self.rconf["allow_unlisted_languages"]):
            self._log([u"Language '%s' is not allowed", language], Logger.CRITICAL)
            raise ValueError("Language code not allowed")

        self._log([u"Synthesizing text: '%s'", text])
        self._log([u"Synthesizing language: '%s'", language])
        self._log([u"Synthesizing to file: '%s'", output_file_path])

        # return zero if text is the empty string
        if len(text) == 0:
            self._log(u"len(text) is zero: returning 0.0")
            return 0.0

        # replace language
        language = self._replace_language(language)
        self._log([u"Using language: '%s'", language])

        result = gf.run_c_extension_with_fallback(
            self._log,
            "cew",
            self._synthesize_single_c_extension,
            self._synthesize_single_pure_python,
            (text, language, output_file_path),
            c_extension=self.rconf["c_ext"]
        )
        return result[0]

    def _synthesize_single_c_extension(self, text, language, output_file_path):
        """
        Synthesize a single text fragment, using cew extension.

        Return the duration of the synthesized text, in seconds.

        :rtype: (bool, (float, ))
        """
        self._log(u"Synthesizing using C extension...")

        self._log(u"Preparing c_text...")
        if gf.PY2:
            # Python 2 => pass byte strings
            c_text = gf.safe_bytes(text)
        else:
            # Python 3 => pass Unicode strings
            c_text = text
        # NOTE language has been replaced already!
        self._log(u"Preparing c_text... done")

        try:
            self._log(u"Importing aeneas.cew...")
            import aeneas.cew.cew
            self._log(u"Importing aeneas.cew... done")
            self._log(u"Calling aeneas.cew...")
            sr, begin, end = aeneas.cew.cew.synthesize_single(
                output_file_path,
                language,
                c_text
            )
            self._log(u"Calling aeneas.cew... done")
        except Exception as exc:
            self._log(u"Calling aeneas.cew... failed")
            self._log(u"An unexpected exception occurred while running cew:", Logger.WARNING)
            self._log([u"%s", exc], Logger.WARNING)
            return (False, None)

        self._log(u"Synthesizing using C extension... done")
        return (True, (end, ))

    def _synthesize_single_pure_python(self, text, language, output_file_path):
        """
        Synthesize a single text fragment, pure Python.

        :rtype: tuple (duration, sample_rate, encoding, data)
        """
        self._log(u"Synthesizing using pure Python...")

        # NOTE language has been replaced already!

        try:
            # call espeak via subprocess
            self._log(u"Calling espeak ...")
            arguments = [self.rconf["espeak_path"], "-v", language, "-w", output_file_path]
            self._log([u"Calling with arguments '%s'", " ".join(arguments)])
            self._log([u"Calling with text '%s'", text])
            proc = subprocess.Popen(
                arguments,
                stdout=subprocess.PIPE,
                stdin=subprocess.PIPE,
                stderr=subprocess.PIPE,
                universal_newlines=True)
            if gf.PY2:
                proc.communicate(input=gf.safe_bytes(text))
            else:
                proc.communicate(input=text)
            proc.stdout.close()
            proc.stdin.close()
            proc.stderr.close()
            self._log(u"Calling espeak ... done")
        except Exception as exc:
            self._log(u"Calling espeak ... failed")
            self._log(u"An unexpected exception occurred while running pure Python code:", Logger.WARNING)
            self._log([u"%s", exc], Logger.WARNING)
            return (False, None)

        # check the file can be read
        if not gf.file_can_be_read(output_file_path):
            self._log([u"Output file '%s' does not exist", output_file_path], Logger.CRITICAL)
            return (False, None)

        # return the duration of the output file
        try:
            audio_file = AudioFileMonoWAVE(
                file_path=output_file_path,
                logger=self.logger
            )
            audio_file.load_data()
            duration = audio_file.audio_length
            sample_rate = audio_file.audio_sample_rate
            encoding = audio_file.audio_format
            data = audio_file.audio_data
            self._log([u"Duration of '%s': %f", output_file_path, duration])
            self._log(u"Synthesizing using pure Python... done")
            return (True, (duration, sample_rate, encoding, data))
        except (AudioFileUnsupportedFormatError, OSError) as exc:
            self._log(u"Error while trying reading the sythesized audio file", Logger.CRITICAL)
            return (False, None)






#!/usr/bin/env python
# coding=utf-8

"""
Compile the Python C Extension for synthesizing text with espeak.

.. versionadded:: 1.3.0
"""

from __future__ import absolute_import
from __future__ import print_function
import sys

from distutils.core import Extension
from distutils.core import setup

__author__ = "Alberto Pettarin"
__copyright__ = """
    Copyright 2012-2013, Alberto Pettarin (www.albertopettarin.it)
    Copyright 2013-2015, ReadBeyond Srl   (www.readbeyond.it)
    Copyright 2015-2016, Alberto Pettarin (www.albertopettarin.it)
    """
__license__ = "GNU AGPL v3"
__version__ = "1.4.1"
__email__ = "aeneas@readbeyond.it"
__status__ = "Production"

CMODULE = Extension("cew", sources=["cew_py.c", "cew_func.c"], libraries=["espeak"])

setup(
    name="cew",
    version="1.4.1",
    description="""
    Python C Extension for synthesizing text with espeak.
    """,
    ext_modules=[CMODULE]
)

print("\n[INFO] Module cew successfully compiled\n")
sys.exit(0)


